"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import IMessage
from zospy.api._ZOSAPI.Common import IMetadata, ISettingsData
from zospy.api._ZOSAPI.Tools.General import IDataDictionary, ZemaxFileTypes

from . import Analysis, Common, Editors, Preferences, SystemData, Tools, Wizards

__all__ = (
    "Analysis",
    "Common",
    "Editors",
    "Preferences",
    "SystemData",
    "Tools",
    "Wizards",
    "IOpticalSystem",
    "IPreferences",
    "ISTARMaterials",
    "ISTARSubsystem",
    "IZAPI_BaseObject",
    "IZOSAPI_Application",
    "IZOSAPI_Callback",
    "IZOSAPI_Connection",
    "LensUpdateMode",
    "LicenseStatusType",
    "OpticalSystemStatusChangedHandler",
    "SessionModes",
    "STARUpdateMode",
    "SystemType",
    "UpdateStatus",
    "ZOSAPI_Mode",
)

class IOpticalSystem:
    @property
    def SystemName(self) -> str: ...
    @SystemName.setter
    def SystemName(self, value: str) -> None: ...
    @property
    def SystemID(self) -> int: ...
    @property
    def Mode(self) -> SystemType: ...
    @property
    def SystemFile(self) -> str: ...
    @property
    def IsNonAxial(self) -> bool: ...
    @property
    def NeedsSave(self) -> bool: ...
    @property
    def SystemData(self) -> ISystemData: ...
    @property
    def LDE(self) -> ILensDataEditor: ...
    @property
    def NCE(self) -> INonSeqEditor: ...
    @property
    def TDE(self) -> IToleranceDataEditor: ...
    @property
    def MFE(self) -> IMeritFunctionEditor: ...
    @property
    def MCE(self) -> IMultiConfigEditor: ...
    @property
    def Analyses(self) -> I_Analyses: ...
    @property
    def Tools(self) -> IOpticalSystemTools: ...
    @property
    def TheApplication(self) -> IZOSAPI_Application: ...
    @property
    def UpdateMode(self) -> LensUpdateMode: ...
    @UpdateMode.setter
    def UpdateMode(self, value: LensUpdateMode) -> None: ...
    @property
    def SessionMode(self) -> SessionModes: ...
    @SessionMode.setter
    def SessionMode(self, value: SessionModes) -> None: ...
    @property
    def STARSubsystem(self) -> ISTARSubsystem: ...
    @property
    def IsProjectDirectory(self) -> bool: ...
    def Close(self, saveIfNeeded: bool) -> bool: ...
    def ConvertToProjectDirectory(self, folderPath: str) -> bool: ...
    def CopySystem(self) -> IOpticalSystem: ...
    def GetCurrentStatus(self) -> str: ...
    def GetMetadata(self) -> IMetadata: ...
    def LoadFile(self, LensFile: str, saveIfNeeded: bool) -> bool: ...
    def MakeNonSequential(self) -> bool: ...
    def MakeSequential(self) -> bool: ...
    def New(self, saveIfNeeded: bool) -> None: ...
    def Save(self) -> None: ...
    def SaveAs(self, fileName: str) -> None: ...
    def TurnOffProjectDirectory(self) -> bool: ...
    def UpdateFileLists(self) -> None: ...
    def UpdateStatus(self) -> str: ...

class IPreferences:
    @property
    def General(self) -> IPreferencesGeneral: ...
    @property
    def Folders(self) -> IPreferencesFolders: ...
    @property
    def Editor(self) -> IPreferencesEditor: ...
    @property
    def IsReadOnly(self) -> bool: ...
    def ResetToDefaults(self) -> None: ...

class ISTARMaterials:
    def GetDNDT(
        self,
        MaterialCatalog: str,
        Material: str,
        Wavelength: float,
        EnvironmentPressure: float,
        EnvironmentTemperature: float,
        MaterialPressure: float,
        MaterialTemperature: float,
        DeltaT: float,
    ) -> float: ...
    def GetRefractiveIndex(
        self,
        MaterialCatalog: str,
        Material: str,
        Wavelength: float,
        EnvironmentPressure: float,
        EnvironmentTemperature: float,
        MaterialPressure: float,
        MaterialTemperature: float,
    ) -> float: ...

class ISTARSubsystem:
    @property
    def UpdateMode(self) -> STARUpdateMode: ...
    @UpdateMode.setter
    def UpdateMode(self, value: STARUpdateMode) -> None: ...
    @property
    def Materials(self) -> ISTARMaterials: ...
    def DoManualUpdate(self) -> None: ...

class IZAPI_BaseObject:
    @property
    def Parent(self) -> IZAPI_BaseObject: ...
    def AddChild(self, child: IZAPI_BaseObject) -> None: ...
    def Disconnect(self) -> None: ...
    def RemoveChild(self, child: IZAPI_BaseObject) -> None: ...

class IZOSAPI_Application:
    @property
    def LicenseStatus(self) -> LicenseStatusType: ...
    @property
    def IsValidLicenseForAPI(self) -> bool: ...
    @property
    def InitializationErrors(self) -> str: ...
    @property
    def InitializationErrorCode(self) -> str: ...
    @property
    def Mode(self) -> ZOSAPI_Mode: ...
    @property
    def SerialCode(self) -> str: ...
    @property
    def OpticStudioVersion(self) -> int: ...
    @property
    def ExpirationYear(self) -> int: ...
    @property
    def ExpirationMonth(self) -> int: ...
    @property
    def ExpirationDay(self) -> int: ...
    @property
    def NumberOfOpticalSystems(self) -> int: ...
    @property
    def PrimarySystem(self) -> IOpticalSystem: ...
    @property
    def NumberOfCPUs(self) -> int: ...
    @property
    def ProgramDir(self) -> str: ...
    @property
    def LensDir(self) -> str: ...
    @property
    def ObjectsDir(self) -> str: ...
    @property
    def GlassDir(self) -> str: ...
    @property
    def ZPLDir(self) -> str: ...
    @property
    def CoatingDir(self) -> str: ...
    @property
    def POPDir(self) -> str: ...
    @property
    def ImagesDir(self) -> str: ...
    @property
    def SolidWorksFilesDir(self) -> str: ...
    @property
    def AutodeskInventorFilesDir(self) -> str: ...
    @property
    def CreoParametricFilesDir(self) -> str: ...
    @property
    def MATLABFilesDir(self) -> str: ...
    @property
    def ScatterDir(self) -> str: ...
    @property
    def UndoDir(self) -> str: ...
    @property
    def SamplesDir(self) -> str: ...
    @property
    def ZemaxDataDir(self) -> str: ...
    @property
    def OperandResults(self) -> IVectorData: ...
    @property
    def OperandArgument1(self) -> float: ...
    @property
    def OperandArgument2(self) -> float: ...
    @property
    def OperandArgument3(self) -> float: ...
    @property
    def OperandArgument4(self) -> float: ...
    @property
    def UserAnalysisData(self) -> IUserAnalysisData: ...
    @property
    def TerminateRequested(self) -> bool: ...
    @property
    def ShowChangesInUI(self) -> bool: ...
    @ShowChangesInUI.setter
    def ShowChangesInUI(self, value: bool) -> None: ...
    @property
    def ProgressMessage(self) -> str: ...
    @ProgressMessage.setter
    def ProgressMessage(self, value: str) -> None: ...
    @property
    def ProgressPercent(self) -> float: ...
    @ProgressPercent.setter
    def ProgressPercent(self, value: float) -> None: ...
    @property
    def CheckForUpdatesStatus(self) -> UpdateStatus: ...
    @property
    def CheckForUpdatesData(self) -> str: ...
    @property
    def CheckForUpdatesVersion(self) -> int: ...
    @property
    def Preferences(self) -> IPreferences: ...
    @property
    def IsLoggingMessages(self) -> bool: ...
    @property
    def ZOSMajorVersion(self) -> int: ...
    @property
    def ZOSMinorVersion(self) -> int: ...
    @property
    def ZOSSPVersion(self) -> int: ...
    @property
    def IsSubscriptionLicense(self) -> bool: ...
    @property
    def IsSTARModuleEnabled(self) -> bool: ...
    def BeginMessageLogging(self) -> bool: ...
    def CheckForUpdates(self) -> None: ...
    def ClearMessageLog(self) -> None: ...
    def CloseApplication(self) -> None: ...
    def CloseSystemAt(self, pos: int, saveIfNeeded: bool) -> bool: ...
    def CopySettingsData(self, from_: ISettingsData, to: ISettingsData) -> bool: ...
    def CreateDataFile(self, fileName: str, type: ZemaxFileTypes, data1: int, data2: int) -> IDataDictionary: ...
    def CreateNewSystem(self, mode: SystemType) -> IOpticalSystem: ...
    def CreateSettings(self) -> ISettingsData: ...
    def CreateSettingsFromParent(self, parent: IZAPI_BaseObject) -> ISettingsData: ...
    def EndMessageLogging(self) -> bool: ...
    def GetDate(self) -> str: ...
    def GetSystemAt(self, pos: int) -> IOpticalSystem: ...
    def LoadCCallback(self, cLib: str, callbackName: str, settings: ISettingsData) -> IZOSAPI_Callback: ...
    def LoadNETCallback(self, fromAssembly: str, typeName: str, settings: ISettingsData) -> IZOSAPI_Callback: ...
    def LoadNewSystem(self, LensFile: str) -> IOpticalSystem: ...
    def OpenDataFile(self, fileName: str) -> IDataDictionary: ...
    def RegisterCOperandCallback(self, cLib: str, callbackName: str, settings: ISettingsData) -> IMessage: ...
    def RegisterNETOperandCallback(self, fromAssembly: str, typeName: str, settings: ISettingsData) -> IMessage: ...
    def RetrieveLogMessages(self) -> str: ...
    def RunCommand(self, command: str) -> str: ...
    def UpdateFileLists(self) -> None: ...

class IZOSAPI_Callback:
    @property
    def Name(self) -> str: ...
    @Name.setter
    def Name(self, value: str) -> None: ...
    @property
    def IsLocal(self) -> bool: ...
    @property
    def Settings(self) -> ISettingsData: ...
    @property
    def TheApp(self) -> IZOSAPI_Application: ...
    def Copy(self, app: IZOSAPI_Application) -> IZOSAPI_Callback: ...
    def Execute(self) -> int: ...

class IZOSAPI_Connection:
    @property
    def IsAlive(self) -> bool: ...
    @property
    def InitializationSettings(self) -> str: ...
    @InitializationSettings.setter
    def InitializationSettings(self, value: str) -> None: ...
    @property
    def IntializationProgress(self) -> list[int]: ...
    @property
    def ConnectionTimeoutSeconds(self) -> float: ...
    @ConnectionTimeoutSeconds.setter
    def ConnectionTimeoutSeconds(self, value: float) -> None: ...
    @property
    def FormattingCulture(self) -> str: ...
    @FormattingCulture.setter
    def FormattingCulture(self, value: str) -> None: ...
    @property
    def PreferencesFile(self) -> str: ...
    @PreferencesFile.setter
    def PreferencesFile(self, value: str) -> None: ...
    def ConnectAsExtension(self, zemaxInstance: int) -> IZOSAPI_Application: ...
    def ConnectToApplication(self) -> IZOSAPI_Application: ...
    def CreateNewApplication(self) -> IZOSAPI_Application: ...
    def CreateZemaxServer(self, applicationName: str) -> IZOSAPI_Application: ...
    def CreateZemaxServerEx(self, applicationName: str, libraryPath: str) -> IZOSAPI_Application: ...
    def GetConfigSetting(self, setting: str) -> str: ...
    def RunCommand(self, command: str) -> str: ...
    def SetConfigSetting(self, setting: str, Value: str) -> None: ...
    def SetCreoInstallPath(self, path: str) -> None: ...

class LensUpdateMode:
    # None = 0
    EditorsOnly = 1
    AllWindows = 2

class LicenseStatusType:
    Unknown = 0
    KeyNotWorking = 1
    NewLicenseNeeded = 2
    StandardEdition = 3
    ProfessionalEdition = 4
    PremiumEdition = 5
    TooManyInstances = 6
    NotAuthorized = 7
    KeyNotFound = 8
    KeyExpired = 9
    Timeout = 10
    InstanceConflict = 11
    OpticsViewer = 12
    OpticStudioHPCEdition = 13
    EnterpriseEdition = 14

class OpticalSystemStatusChangedHandler:
    def __init__(self, object: Object, method: IntPtr): ...
    @property
    def Method(self) -> MethodInfo: ...
    @property
    def Target(self) -> Object: ...
    def BeginInvoke(self, systemId: int, newStatus: str, callback: AsyncCallback, object: Object) -> IAsyncResult: ...
    def EndInvoke(self, result: IAsyncResult) -> None: ...
    def Invoke(self, systemId: int, newStatus: str) -> None: ...

class SessionModes:
    FromPreferences = 0
    SessionOn = 1
    SessionOff = 2

class STARUpdateMode:
    Normal = 0
    Suspended = 1

class SystemType:
    Sequential = 0
    NonSequential = 1

class UpdateStatus:
    NotChecked = 0
    UpToDate = 1
    AvailableEligible = 2
    AvailableIneligible = 3
    CheckFailed = -2
    NotSupported = -1

class ZOSAPI_Mode:
    Server = 0
    Operand = 1
    Plugin = 2
    UserAnalysis = 3
    UserAnalysisSettings = 4



"""Manage the connection with OpticStudio and interact with the optical system.

This module provides two classes:

- `ZOS` is used to connect to OpticStudio and manage the connection.
- `OpticStudioSystem` is used to interact with the optical system.

Examples
--------
Connect to OpticStudio and create a new optical system:

>>> import zospy as zp
>>> zos = zp.ZOS()
>>> oss = zos.connect()
>>> oss.make_sequential()
>>> oss.SystemName = "New System"
>>> oss.LDE.InsertNewSurfaceAt(1)
"""

from __future__ import annotations

import locale
import logging
import warnings
import weakref
from sys import version_info
from typing import TYPE_CHECKING, Literal
from weakref import WeakValueDictionary

from semver.version import Version

from zospy.api import _ZOSAPI, constants
from zospy.api.apisupport import load_zosapi, load_zosapi_nethelper
from zospy.utils.pyutils import abspath

if TYPE_CHECKING:
    from os import PathLike

__all__ = ("OpticStudioSystem", "ZOS")

logger = logging.getLogger(__name__)


class OpticStudioSystem:
    """Wrapper for OpticStudio System instances."""

    def __init__(self, zos_instance, system_instance):
        """Initiate the OpticStudioSystem.

        Parameters
        ----------
        zos_instance : ZOS
            A ZOS instance
        system_instance : ZOS.Application.PrimarySystem
            A PrimarySystem instance obtained from the zos_instance.
        """
        # Use weakref to make sure that the ZOS instance is not kept alive by the OpticStudioSystem instance
        self.ZOS: ZOS = weakref.proxy(zos_instance)

        self._System: _ZOSAPI.IOpticalSystem = system_instance
        self._OpenFile = None

    @property
    def SystemName(self) -> str:  # noqa: N802
        """Name of the current optical system."""
        return self._System.SystemName

    @SystemName.setter
    def SystemName(self, value: str):  # noqa: N802
        self._System.SystemName = value

    @property
    def SystemID(self) -> int:  # noqa: N802
        """Unique identifier for the optical system.

        This identifier can be used to differentiate between multiple `OpticStudioSystem` instances.

        Examples
        --------
        Create two different optical systems in a single instance of OpticStudio:

        >>> import zospy as zp
        >>> zos = zp.ZOS()
        >>> oss1 = zos.connect()
        >>> oss2 = zos.create_new_system()
        >>> oss1.SystemID != oss2.SystemID
        True
        """
        return self._System.SystemID

    @property
    def Mode(self) -> str:  # noqa: N802
        """Mode of the optical system. Either "Sequential" or "NonSequential"."""
        return str(self._System.Mode)

    @property
    def SystemFile(self) -> str:  # noqa: N802
        """File path to the current optical system."""
        return self._System.SystemFile

    @property
    def IsNonAxial(self) -> bool:  # noqa: N802
        """Indicates whether the optical system is axial and sequential.

        `True` if the system is non-axial, `False` otherwise.
        """
        return self._System.IsNonAxial

    @property
    def NeedsSave(self) -> bool:  # noqa: N802
        """Indicates if the optical system contains unsaved changes."""
        return self._System.NeedsSave

    @property
    def SystemData(self) -> _ZOSAPI.SystemData.ISystemData:  # noqa: N802
        """Data for configuring everything in the system explorer.

        Examples
        --------
        Change the aperture type to "FloatByStopSize":

        >>> oss.SystemData.Aperture.ApertureType = (
        ...     zp.constants.SystemData.ZemaxApertureType.FloatByStopSize
        ... )

        Add a wavelength of 543 nm with weight 1:

        >>> oss.SystemData.Wavelengths.AddWavelength(0.543, 1)
        """
        return self._System.SystemData

    @property
    def LDE(self) -> _ZOSAPI.Editors.LDE.ILensDataEditor:  # noqa: N802
        """Lens Data Editor."""
        return self._System.LDE

    @property
    def NCE(self) -> _ZOSAPI.Editors.NCE.INonSeqEditor:  # noqa: N802
        """Non-Sequential Component Editor."""
        return self._System.NCE

    @property
    def MFE(self) -> _ZOSAPI.Editors.MFE.IMeritFunctionEditor:  # noqa: N802
        """Merit Function Editor."""
        return self._System.MFE

    @property
    def TDE(self) -> _ZOSAPI.Editors.TDE.IToleranceDataEditor:  # noqa: N802
        """Tolerance Data Editor."""
        return self._System.TDE

    @property
    def MCE(self) -> _ZOSAPI.Editors.MCE.IMultiConfigEditor:  # noqa: N802
        """Multi-Configuration Editor."""
        return self._System.MCE

    @property
    def Analyses(self) -> _ZOSAPI.Analysis.I_Analyses:  # noqa: N802
        """Analyses for the current system."""
        return self._System.Analyses

    @property
    def Tools(self) -> _ZOSAPI.Tools.IOpticalSystemTools:  # noqa: N802
        """Interface to run various tools on the optical system."""
        return self._System.Tools

    @property
    def TheApplication(self) -> _ZOSAPI.IZOSAPI_Application:  # noqa: N802
        """Application in which the optical system is opened."""
        return self._System.TheApplication

    @property
    def LensUpdateMode(self) -> str:  # noqa: N802
        """Lens update mode of the optical system.

        Possible values are ['None', 'EditorsOnly', 'AllWindows'] or `zospy.constants.LensUpdateMode`.
        """
        return str(self._System.UpdateMode)

    @LensUpdateMode.setter
    def LensUpdateMode(self, value: constants.LensUpdateMode | str):  # noqa: N802
        self._System.UpdateMode = value

    @property
    def SessionModes(self) -> str:  # noqa: N802
        """Session mode of the optical system.

        Possible values are ['FromPreferences', 'SessionOn', 'SessionOff'], or `zospy.constants.SessionModes`.
        """
        return str(self._System.SessionMode)

    @SessionModes.setter
    def SessionModes(self, value: constants.SessionModes | str):  # noqa: N802
        self._System.SessionMode = constants.process_constant(constants.SessionModes, value)

    def get_current_status(self) -> str:
        """Get the last status of the optical system. If null or the length is 0, then the system has no errors.

        Returns
        -------
        str
            Current status of the optical system.
        """
        return self._System.GetCurrentStatus()

    def update_status(self) -> str:
        """Force an update of the system, and returns the current status.

        Returns
        -------
        str
            Current status of the optical system.
        """
        return self._System.UpdateStatus()

    def make_sequential(self) -> bool:
        """Set the optical system to sequential mode if it is not already."""
        return self._System.MakeSequential()

    def make_nonsequential(self) -> bool:
        """Set the optical system to non-sequential mode if it is not already."""
        return self._System.MakeNonSequential()

    def load(self, filepath: str | PathLike, *, saveifneeded: bool = False):
        """Load an earlier defined Zemax file.

        Parameters
        ----------
        filepath : str | PathLike
            Path to the file that should be loaded
        saveifneeded : bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        filepath = abspath(filepath)

        logger.debug(f"Opening {filepath} with SaveIfNeeded set to {saveifneeded}")

        self._System.LoadFile(filepath, saveifneeded)
        self._OpenFile = filepath

        logger.info(f"Opened {filepath}")

    def new(self, *, saveifneeded: bool = False):
        """Create a new session file.

        Parameters
        ----------
        saveifneeded : bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        logger.debug("Creating new file")

        self._System.New(saveifneeded)
        self._OpenFile = None

        logger.info("Opened new file")

    def save_as(self, filepath: str | PathLike):
        """Save the current session under a specified name.

        Parameters
        ----------
        filepath : str | PathLike
            The filepath where the system should be saved. Note that a partial path or relative path does not work.
        """
        filepath = abspath(filepath, check_directory_only=True)

        logger.debug(f"Saving open session as {filepath}")

        self._System.SaveAs(filepath)
        self._OpenFile = filepath

        logger.info(f"File saved as {filepath}")

    def save(self) -> bool:
        """Save the current OpticStudio session.

        If the file name for the current session is not known (e.g. when a new file was created), a warning is raised.
        On these occurences, save_as() should be used once.

        Returns (bool): A boolean indicating if the saving was successful.
        """
        logger.debug("Saving file")

        if self._OpenFile is None:
            warnings.warn(
                "The optical system has not been saved because no file name has been defined for the current system."
                "Please use the 'save_as' function before using 'save'."
            )
            logger.critical("Could not save file because the path has not been set using save_as")

        self._System.Save()
        logger.info("File saved")
        return True

    def close(self, *, saveifneeded: bool = False) -> bool:
        """Close the optical system. Only works on secondary systems (See OpticStudio documentation).

        Parameters
        ----------
        saveifneeded : bool
            Defines if the current file is saved before opening the new file. Defaults to False.
        """
        return self._System.Close(saveifneeded)

    def copy_system(self) -> OpticStudioSystem:
        """Copy the current OpticStudioSystem instance.

        Returns
        -------
        zospy.zpcore.OpticStudioSystem
            A ZOSPy OpticStudioSystem instance. Should be sequential.
        """
        return OpticStudioSystem(self.ZOS, self._System.CopySystem())

    def _ensure_correct_mode(self, required: str):
        """Ensure that the system is in the required type.

        Parameters
        ----------
        required : str
            The required system type. Either 'Sequential' or 'NonSequential'.

        Returns
        -------
        None

        Raises
        ------
        ValueError:
            When the system is in the incorrect type
        """
        logger.debug("Ensuring correct mode")
        if self.Mode != required:
            err = f"Incorrect system type. System is in {self.Mode} mode but {required} mode is required"
            logger.critical(err)
            raise TypeError(err)
        logger.debug("System is in correct mode")
        # TODO: Eveluate what happens in 'mixed mode'

    def __del__(self):
        """Close the optical system when the instance is deleted."""
        logger.debug("Closing connections with Zemax OpticStudio")


class ZOS:
    """A Communication instance for Zemax OpticStudio.

    This class manages the connection between Python and Zemax OpticStudio through .NET, and controls OpticStudio
    instances. Only one instance of `ZOS` can exist at any time. If a second instance is attempted to be created, the
    existing instance is returned.

    Parameters
    ----------
    preload : bool
        A boolean indicating if nested namespaces should be preloaded upon initiating ZOS. Defaults to False.
    zosapi_nethelper : str | None
        Optional filepath to the ZOSAPI_NetHelper dll that is required to connect to OpticStudio. If None, the
        Windows registry will be used to find the ZOSAPI_NetHelper dll. Defaults to `None`.
    opticstudio_directory : str, optional
        Path to the directory containing the OpticStudio DLLs. If `None`, the directory will be acquired through the
        zosapi_nethelper. Note that either the zosapi_nethelper or the opticstudio_directory has to be specified,
        not both.

    Attributes
    ----------
    ZOSAPI : None | netModuleObject
        The ZOSAPI interface once loaded, else `None`.
    ZOSAPI_NetHelper : None | netModuleObject
        The ZOSAPI_NetHelper interface once loaded, else `None`.

    Examples
    --------
    Preferred methods:

    1. Connecting as extension:

        Make sure OpticStudio is running and the "Interactive Extension" mode in the programming tab is active.

        >>> import zospy as zp
        >>> zos = zp.ZOS()
        >>> oss = zos.connect(mode="extension")

    2. Launching OpticStudio in standalone mode:

        >>> import zospy as zp
        >>> zos = zp.ZOS()
        >>> oss = zos.connect(mode="standalone")
    """

    _instances: WeakValueDictionary = WeakValueDictionary()

    def __new__(cls, *args, **kwargs):  # noqa: ARG003
        """Ensure that only one instance of ZOS exists at any time.

        If a ZOS instance already exists, the existing instance is returned. If not, a new instance is created.
        """
        if cls not in cls._instances:
            instance = super().__new__(cls)
            instance.__initialized = False  # noqa: SLF001
            cls._instances[cls] = instance
        else:
            warnings.warn("Only a single instance of ZOS can exist at any time. Returning existing instance.")

        return cls._instances[cls]

    def __init__(
        self, *, preload: bool = False, zosapi_nethelper: str | None = None, opticstudio_directory: str | None = None
    ):
        """Initiate the OpticStudio API.

        The ZOS instance can subsequently be used to connect to OpticStudio. See the examples in the class docstring for
        more information.

        Parameters
        ----------
        preload : bool
            A boolean indicating if nested namespaces should be preloaded upon initiating ZOS. Defaults to False.
        zosapi_nethelper : str, optional
            Path to the ZOSAPI_NetHelper dll that is required to connect to OpticStudio. If `None`, the
            Windows registry will be used to find the ZOSAPI_NetHelper dll. Defaults to `None`.
        opticstudio_directory : str, optional
            Path to the directory containing the OpticStudio DLLs. If `None`, the directory will be acquired through the
            zosapi_nethelper. Note that either the zosapi_nethelper or the opticstudio_directory has to be specified,
            not both.
        """
        if self.__initialized:
            logger.debug("ZOS instance already initialized")
            return

        logger.debug("Initializing ZOS instance")

        self.ZOSAPI: _ZOSAPI = None
        self.ZOSAPI_NetHelper = None
        self.Connection: _ZOSAPI.IZOSAPI_Connection | None = None
        self.Application: _ZOSAPI.IZOSAPI_Application | None = None

        # Register the instance and create finalizers to tear down the ZOS instance when deleted
        weakref.finalize(self, self.disconnect)

        logger.info("ZOS instance initialized")

        self._wakeup(preload=preload, zosapi_nethelper=zosapi_nethelper, opticstudio_directory=opticstudio_directory)

        self.__initialized = True

    def _wakeup(
        self, *, preload: bool = False, zosapi_nethelper: str | None = None, opticstudio_directory: str | None = None
    ):
        """Wake the ZOSAPI instance.

        The parameters are passed to `ZOS._load_zos_dlls`.

        Parameters
        ----------
        preload : bool
            A boolean indicating if nested namespaces should be preloaded.
        zosapi_nethelper : str, optional
            File path to the ZOSAPI_NetHelper dll, if `None`, the Windows registry will be used to find
            ZOSAPI_NetHelper dll. Defaults to `None`.
        opticstudio_directory : str, optional
            Path to the directory containing the OpticStudio DLLs. If `None`, the directory will be acquired through the
            zosapi_nethelper. Note that either the zosapi_nethelper or the opticstudio_directory has to be specified,
            not both.
        """
        if self.Connection is None:
            self._load_zos_dlls(
                preload=preload, zosapi_nethelper=zosapi_nethelper, opticstudio_directory=opticstudio_directory
            )
            self._assign_connection()

    def _load_zos_dlls(
        self, *, preload: bool = False, zosapi_nethelper: str | None = None, opticstudio_directory: str | None = None
    ):
        """Load the ZOS-API DLLs and makes them available for usage through ZOS.ZOSAPI and ZOS.ZOSAPI_NetHelper.

        As there are many nested namespaces in the ZOSAPI DLLs that are always available after import but not
        preloaded, it can be chosen to do so. This should only be done for development reasons, as it likely slows
        down the code.

        If multiple versions of OpticStudio are installed on the same system, a specific version can be used by
        manually specifying the path to the OpticStudio installation directory with `opticstudio_directory`.

        Parameters
        ----------
        preload : bool
            A boolean indicating if nested namespaces should be preloaded.
        zosapi_nethelper : str, optional
            Path to the ZOSAPI_NetHelper dll, if `None`, the Windows registry will be used to find
            ZOSAPI_NetHelper dll. Defaults to `None`.
        opticstudio_directory : str, optional
            Path to the directory containing the OpticStudio DLLs. If `None`, the directory will be acquired through the
            zosapi_nethelper. Note that either the zosapi_nethelper or the opticstudio_directory has to be specified,
            not both.

        Raises
        ------
        FileNotFoundError
            An error occurred in locating one of the DLLs.
        ModuleNotFoundError
            One of the nested namespaces cannot be found. This error can only occur with preload set to True.
        ValueError
            Both `zosapi_nethelper` and `opticstudio_directory` are specified.
        """
        if zosapi_nethelper is not None and opticstudio_directory is not None:
            raise ValueError("Only one of `zosapi_nethelper` and `opticstudio_directory` may be specified.")

        logger.debug(f"Loading ZOS DLLs with preload set to {preload}")

        if opticstudio_directory is not None:
            self.ZOSAPI = load_zosapi(zemaxdirectory=opticstudio_directory, preload=preload)
        else:
            self.ZOSAPI_NetHelper = load_zosapi_nethelper(filepath=zosapi_nethelper, preload=preload)
            self.ZOSAPI = load_zosapi(zosapi_nethelper=self.ZOSAPI_NetHelper, preload=preload)

    def _assign_connection(self):
        """Assign the ZOSAPI Connection to self.Connection."""
        if not self.Connection:
            logger.debug("Assigning ZOSAPI_Connection() to self.Connection")
            self.Connection = self.ZOSAPI.ZOSAPI_Connection()
        else:
            logger.debug("ZOSAPI_Connection() already assigned self.Connection")

    def connect(
        self,
        mode: Literal["standalone", "extension"] = "standalone",
        instance_number: int | None = None,
        *,
        message_logging: bool = True,
    ) -> OpticStudioSystem:
        """Connect to Zemax OpticStudio.

        The application will be assigned to ZOS.Application.

        Parameters
        ----------
        mode : str
            Connection mode, either "standalone" or "extension". Default is "standalone".
            In standalone mode, ZOSPy initializes a new, invisible OpticStudio instance and connects to this instance.
            In extension mode, ZOSPy connects to an already open instance of OpticStudio in "Interactive Extension"
            mode.
        instance_number : int, optional
            An integer to specify the number of the instance used. Only applicable in extension mode.
        message_logging : bool, optional
            If `True`, OpticStudio's message logging will be enabled after establishing a connection.

        Returns
        -------
        OpticStudioSystem
            The primary optical system of the connected OpticStudio instance.
        """
        if mode != "extension" and instance_number is not None:
            warnings.warn("Instance number is only used in extension mode.")

        self._assign_connection()

        if self.Connection.IsAlive:  # TODO ensure no memory leak
            raise RuntimeError("Only one Zemax application can exist within runtime")

        if mode == "standalone":
            self.Application = self.Connection.CreateNewApplication()
        elif mode == "extension":
            self.Application = self.Connection.ConnectAsExtension(instance_number or 0)
        else:
            raise ValueError(f"Invalid connection mode {mode}")

        if not self.Application.IsValidLicenseForAPI:
            logger.critical("OpticStudio Licence is not valid for API, connection not established")
            raise ConnectionRefusedError("OpticStudio Licence is not valid for API, connection not established")

        if message_logging:
            self.Application.BeginMessageLogging()

        return self.get_primary_system()

    def disconnect(self):
        """Disconnect from the connected OpticStudio instance.

        When connected in standalone mode, this closes the standalone OpticStudio instance. When connected in extension
        mode, this closes the connection but keeps the OpticStudio instance open.
        """
        logger.debug("Disconnecting from OpticStudio")

        if self.Application is not None:
            self.Application.CloseApplication()
            self.Application = None

        logger.info("Disconnected from OpticStudio")

    def create_new_system(self, system_mode: constants.SystemType | str = "Sequential") -> OpticStudioSystem:
        """Create a new OpticStudioSystem.

        This works only if ZOSPy is connected to a standalone application.

        Parameters
        ----------
        system_mode : constants.SystemType | str
            The mode of the new system, must be one of "Sequential", "NonSequential". Defaults to "Sequential".

        Returns
        -------
        OpticStudioSystem
            The new OpticStudioSystem.
        """
        if self.Application.Mode == constants.ZOSAPI_Mode.Server:
            new_system = self.Application.CreateNewSystem(constants.process_constant(constants.SystemType, system_mode))
            return OpticStudioSystem(zos_instance=self, system_instance=new_system)

        # TODO: Check if this is really true
        raise ValueError("Can only create a new system when using a standalone connection.")

    def get_primary_system(self) -> OpticStudioSystem:
        """Get the primary optical system.

        Returns
        -------
        OpticStudioSystem
            Primary optical system.
        """
        optic_studio_system = OpticStudioSystem(zos_instance=self, system_instance=self.Application.PrimarySystem)

        # Automatically populate _OpenFile when connecting in extension mode, to prevent unnecessary errors when
        # calling save
        if self.Application.Mode == constants.ZOSAPI_Mode.Plugin:
            optic_studio_system._OpenFile = optic_studio_system.SystemFile  # noqa: SLF001

        return optic_studio_system

    def get_system(self, pos: int = 0) -> OpticStudioSystem:
        """Get the optical system at the specified position.

        Parameters
        ----------
        pos : int
            Index of the optical system. If `0`, the primary system is returned.

        Returns
        -------
        OpticStudioSystem
            Optical system at position `pos`.
        """
        optic_studio_system = self.Application.GetSystemAt(pos)
        return OpticStudioSystem(zos_instance=self, system_instance=optic_studio_system)

    def get_txtfile_encoding(self) -> str:
        """Determine the encoding used to write text files in OpticStudio.

        Returns
        -------
        str
            The encoding used for textfiles by OpticStudio

        Raises
        ------
        RuntimeError
            When ZOS does not have a connection to the OpticStudio application
        NotImplementedError
            When the ZOS.Application.Preferences.General.TXTFileEncoding is not one of ["Unicode", "ANSI"]
        """
        if self.Application is None:
            raise RuntimeError("ZOS.get_txtfile_encoding requires a live connection to the OpticStudio application.")

        if str(self.Application.Preferences.General.TXTFileEncoding) == "Unicode":
            return "UTF-16-le"
        if str(self.Application.Preferences.General.TXTFileEncoding) == "ANSI":
            if version_info < (3, 11):
                return locale.getpreferredencoding(False)

            # Python 3.11 introduced locale.getencoding, which returns the system preferred encoding also if Python's
            # UTF-8 mode is enabled
            return locale.getencoding()
        raise NotImplementedError(
            f"ZOSPy cannot handle encoding {self.Application.Preferences.General.TXTFileEncoding!s}"
        )

    def retrieve_logs(self) -> str:
        """Retrieve messages logged by OpticStudio.

        Returns
        -------
        str
            Messages logged by OpticStudio.
        """
        return self.Application.RetrieveLogMessages()

    @property
    def version(self) -> Version:
        """Returns the OpticStudio version as Version object."""
        return Version(
            major=self.Application.ZOSMajorVersion,
            minor=self.Application.ZOSMinorVersion,
            patch=self.Application.ZOSSPVersion,
        )

    @classmethod
    def get_instance(cls) -> ZOS | None:
        """Get the current instance of ZOS.

        Unlike `__new__`, this method does not create a new instance if none exists.

        Returns
        -------
        ZOS | None
            The current instance of ZOS, or `None` if no instance exists.
        """
        return cls._instances.get(cls)



"""Transformers and helper classes for parsing OpticStudio analysis output.

Provides the `ZospyTransformer` class for transforming common OpticStudio constructs into dictionaries.
"""

from __future__ import annotations

from itertools import groupby
from typing import Any, NamedTuple, TypedDict, TypeVar

from lark import Discard, Transformer

from zospy.utils.pyutils import atox

FieldValue = TypeVar("FieldValue")


class SimpleField(NamedTuple):
    """A simple field with a name and a value."""

    name: str
    value: FieldValue


class ParametricField(NamedTuple):
    """A field with parameters and a value.

    Should be used as a value for a `SimpleField`.
    """

    parameters: int | float | tuple[int | float, ...]
    value: FieldValue


class UnitField(TypedDict):
    """A field with a unit.

    Should be used as a value for a `SimpleField`.
    """

    value: float
    unit: str


def group_parametric_fields(
    parametric_fields: list[SimpleField[ParametricField]],
) -> dict[str, dict[list[int | float], Any]]:
    """Convert a list of parametric fields into a dictionary of dictionaries."""
    result = {}

    for name, fields in groupby(parametric_fields, lambda field: field.name):
        result[name] = {field.value.parameters: field.value.value for field in fields}

    return result


class ZospyTransformer(Transformer):
    """Parse tree transformations for common constructs in OpticStudio analysis output."""

    DATE = str
    WORD = str

    def INT(self, i: str) -> int:  # noqa: N802
        """Integer number."""
        return atox(i, int)

    def UINT(self, i: str) -> int:  # noqa: N802
        """Unsigned integer number."""
        return atox(i, int)

    def FLOAT(self, f: str) -> float:  # noqa: N802
        """Floating point number with localized decimal separator."""
        return atox(f, float)

    multi_string = " ".join

    def string_list(self, args) -> list[str]:
        """List of whitespace-separated strings."""
        return list(map(str, args))

    list = list

    def dict(self, args):
        """Transform a key-value mapping."""
        keys = (f.name for f in args)

        result = {}

        for is_parametric, fields in groupby(args, lambda field: isinstance(field.value, ParametricField)):
            if is_parametric:
                result.update(group_parametric_fields(fields))
            else:
                result.update({field.name: field.value for field in fields})

        return {k: result[k] for k in keys}

    def start(self, args):
        """Transform the root of the parse tree."""
        return dict(args)

    def table(self, args):
        """Transform a table with a header and one or more rows."""
        header, *rows = args

        return (header, rows)

    def field_group(self, args):
        """Transform a group of fields under a common key to a SimpleField."""
        name, fields = args

        return SimpleField(name, fields)

    def simple_field(self, args) -> SimpleField:
        """Transform a simple field with a name and a value."""
        if len(args) == 2:
            name, value = args
            return SimpleField(str(name), value)

        if len(args) == 1:
            return SimpleField(str(args[0]), None)

        return args

    def parametric_unit_field(self, args) -> SimpleField[ParametricField[UnitField]]:
        """Transform a field with parameters and a unit."""
        name, parameters, value, unit = args

        return SimpleField(str(name), ParametricField(parameters, UnitField(value=value, unit=unit)))

    def unit_field(self, args) -> SimpleField[UnitField]:
        """Transform a field with a unit."""
        name, value, unit = args

        return SimpleField(str(name), UnitField(value=value, unit=unit))

    def unit(self, args) -> str:
        """Transform a unit of measurement."""
        return " ".join(args)

    def parametric_field(self, args) -> SimpleField[ParametricField]:
        """Transform a field with parameters."""
        name, parameters, value = args

        return SimpleField(str(name), ParametricField(parameters, value))

    def field_name(self, name) -> str:
        """Transform a field name."""
        return " ".join(name)

    def field_value(self, value):
        """Transform a field value."""
        (value,) = value
        return value

    def field_parameters(self, args):
        """Transform field parameters."""
        if len(args) == 1:
            return args[0]

        return tuple(args)

    def text(self, args):  # noqa: ARG002
        """Discard text that can be ignored."""
        return Discard



from __future__ import annotations

from operator import attrgetter
from string import Template
from typing import Callable, Literal

from jsonata import Jsonata

import zospy as zp


class JSONataExpressionTemplate(Template):
    delimiter = "%"


class DataConversion:
    DEFAULT = "$.data.Data.data"
    DATAFRAME = DEFAULT


class SettingsConversion:
    DEFAULT = "$each($.data.Settings.data, function($v, $k) {{ $camelToSnake($k): $v}}) ~> $merge"


class ConversionExpression(Jsonata):
    EXPRESSION = JSONataExpressionTemplate(r"""
(
    /* Convert CamelCase variables to snake_case */
    $camelToSnake := function($s) {(
        /* Split groups and separate with underscore */
        $snake := $replace($s, /(?<!^)((?<=[a-z])[A-Z]|[A-Z](?=[a-z])|(?<=[A-Z])[0-9])/, "_$1").$lowercase();
        /* Remove dashes, spaces, and make lowercase */
        $snake.$replace(/[\(\)]/, "").$replace(/[\-\s]/, "_").$replace(/_+/, "_")
    )};

    {
        "data": %data,
        "settings": %settings,
        "metadata": $each($.data.MetaData.data, function($v, $k){
            { $k: $v.data ? $v.data : $v }
        }) ~> $merge,
        "header": $.data.HeaderData,
        "messages": $.data.Messages
    };
)
""")

    def __init__(self, *, data=DataConversion.DEFAULT, settings=SettingsConversion.DEFAULT):
        super().__init__(self.EXPRESSION.substitute(data=data, settings=settings))


class AnalysisDataConverter:
    def __init__(
        self,
        old_analysis: str,
        new_analysis: str,
        settings_class: str,
        module: str,
        data_type: Literal["dataframe", "dataclass"],
        data_class: str | None = None,
        data_conversion: str = DataConversion.DEFAULT,
        settings_conversion: str = SettingsConversion.DEFAULT,
        settings_replace_keys: dict[str, str] | None = None,
        settings_convert_constants: dict[str, str] | None = None,
        postprocess: Callable[[dict], None] | None = None,
    ):
        """Initialize a converter for analysis data.

        Parameters
        ----------
        old_analysis : str
            The name of the old analysis.
        new_analysis : str
            The name of the new analysis.
        settings_class : str
            The name of the settings class.
        module : str
            The module where the settings class is defined.
        data_type : Literal["dataframe"]
            The type of the data.
        data_conversion : str, optional
            The JSONata expression to convert the data, by default returns the data as is.
        settings_replace_keys : dict[str, str], optional
            A dictionary to replace keys in the settings, by default None. Should be in the form {old_key: new_key}.
            This can be used to rename keys in the settings to match the new analysis, in case the conversion to
            snake_case does not result in the correct key.
        settings_convert_constants : list[str], optional
            A mapping of settings keys to constants that need to be converted from an integer to a string.
            Should be in the form {key: constant_name}.
        """
        if data_type == "dataclass" and data_class is None:
            raise ValueError("data_class is required for data_type 'dataclass'")

        self.old_analysis = old_analysis
        self.new_analysis = new_analysis
        self.settings_class = settings_class
        self.module = module
        self.data_type = data_type
        self.data_class = data_class
        self.data_conversion = data_conversion
        self.settings_conversion = settings_conversion
        self.settings_replace_keys = settings_replace_keys
        self.settings_convert_constants = settings_convert_constants
        self.postprocess = postprocess

    def add_metadata(self, data: dict):
        metadata = {
            "__analysis_data__": {
                "data_type": self.data_type,
            },
            "__analysis_settings__": {
                "data_type": "dataclass",
                "name": self.settings_class,
                "module": self.module,
            },
        }

        if self.data_type == "dataclass":
            metadata["__analysis_data__"]["name"] = self.data_class
            metadata["__analysis_data__"]["module"] = self.module

        data.update(metadata)

    def replace_settings_keys(self, settings: dict):
        if self.settings_replace_keys is not None:
            for old_key, new_key in self.settings_replace_keys.items():
                if old_key in settings:
                    settings[new_key] = settings.pop(old_key)

    def convert_settings_constants(self, settings: dict | None):
        if self.settings_convert_constants is not None and settings is not None:
            zos = zp.ZOS()  # noqa: F841

            for key, constant in self.settings_convert_constants.items():
                zospy_constant = attrgetter(constant)(zp.constants)

                try:
                    settings[key] = zp.constants.get_constantname_by_value(zospy_constant, int(settings[key]))
                except ValueError as e:
                    print(f"Error converting constant {key}: {e}")

    def convert_data(self, data: dict):
        converter = ConversionExpression(data=self.data_conversion, settings=self.settings_conversion)
        converted_data = converter.evaluate(data)
        self.replace_settings_keys(converted_data["settings"])
        # self.convert_settings_constants(converted_data["settings"])
        self.add_metadata(converted_data)

        if self.postprocess is not None:
            self.postprocess(converted_data)

        return converted_data



import pytest

from zospy.analyses.old.reports import (
    cardinal_points,
    cardinal_points_fromcfg,
    surface_data,
    surface_data_fromcfg,
    system_data,
)

pytestmark = pytest.mark.old_analyses


class TestSurfaceData:
    def test_can_run_surface_data(self, simple_system):
        result = surface_data(simple_system, 3)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = surface_data(simple_system, 3)

        assert result.from_json(result.to_json())

    def test_can_run_surface_data_fromcfg(self, simple_system, cfg_file):
        result = surface_data_fromcfg(simple_system, str(cfg_file))

        assert result.Data is not None

    def test_surface_data_fromcfg_loads_config_correctly(self, simple_system, cfg_file):
        result = surface_data_fromcfg(simple_system, str(cfg_file))

        # Check if the correct surface thickness is returned
        assert result.Data.Surface.Thickness == 19.792

    def test_surface_data_returns_correct_result(self, simple_system, expected_data):
        result = surface_data(simple_system, surface=2)

        assert all(result.Data.Surface == expected_data.Data.Surface)
        assert all(result.Data.IndexOfRefraction == expected_data.Data.IndexOfRefraction)
        assert all(result.Data.IndexOfRefractionPerWavelength == expected_data.Data.IndexOfRefractionPerWavelength)
        assert all(result.Data.Other == expected_data.Data.Other)
        assert all(result.Data.SurfacePowerAsSituated == expected_data.Data.SurfacePowerAsSituated)
        assert all(result.Data.SurfacePowerInAir == expected_data.Data.SurfacePowerInAir)

    def test_surface_data_matches_reference_data(self, simple_system, reference_data):
        result = surface_data(simple_system, surface=2)

        assert all(result.Data.Surface == reference_data.Data.Surface)
        assert all(result.Data.IndexOfRefraction == reference_data.Data.IndexOfRefraction)
        assert all(result.Data.IndexOfRefractionPerWavelength == reference_data.Data.IndexOfRefractionPerWavelength)
        assert all(result.Data.Other == reference_data.Data.Other)
        assert all(result.Data.SurfacePowerAsSituated == reference_data.Data.SurfacePowerAsSituated)
        assert all(result.Data.SurfacePowerInAir == reference_data.Data.SurfacePowerInAir)


class TestSystemData:
    def test_can_run_system_data(self, simple_system):
        result = system_data(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = system_data(simple_system)

        assert result.from_json(result.to_json())

    def test_system_data_returns_correct_result(self, simple_system, expected_data):
        result = system_data(simple_system)

        assert all(result.Data.Fields.Data == expected_data.Data.Fields.Data)
        assert all(result.Data.Fields.Info == expected_data.Data.Fields.Info)
        assert all(result.Data.GeneralLensData == expected_data.Data.GeneralLensData)
        assert all(result.Data.PredictedCoordinateABCDMatrix == expected_data.Data.PredictedCoordinateABCDMatrix)
        assert all(result.Data.VignettingFactors == expected_data.Data.VignettingFactors)
        assert all(result.Data.Wavelengths.Data == expected_data.Data.Wavelengths.Data)
        assert all(result.Data.Wavelengths.Info == expected_data.Data.Wavelengths.Info)

    def test_system_data_matches_reference_data(self, simple_system, reference_data):
        result = system_data(simple_system)

        assert all(result.Data.Fields.Data == reference_data.Data.Fields.Data)
        assert all(result.Data.Fields.Info == reference_data.Data.Fields.Info)
        assert all(result.Data.GeneralLensData == reference_data.Data.GeneralLensData)
        assert all(result.Data.PredictedCoordinateABCDMatrix == reference_data.Data.PredictedCoordinateABCDMatrix)
        assert all(result.Data.VignettingFactors == reference_data.Data.VignettingFactors)
        assert all(result.Data.Wavelengths.Data == reference_data.Data.Wavelengths.Data)
        assert all(result.Data.Wavelengths.Info == reference_data.Data.Wavelengths.Info)


class TestCardinalPoints:
    def test_can_run_cardinal_points(self, simple_system):
        result = cardinal_points(simple_system, 2, 3)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = cardinal_points(simple_system, 2, 3)

        assert result.from_json(result.to_json())

    def test_can_run_cardinal_points_fromcfg(self, simple_system, cfg_file):
        result = cardinal_points_fromcfg(simple_system, str(cfg_file))

        assert result.Data is not None

    def test_cardinal_points_fromcfg_loads_config_correctly(self, simple_system, cfg_file):
        result = cardinal_points_fromcfg(simple_system, str(cfg_file))

        assert result.Data.Surface["Starting surface"] == 2
        assert result.Data.Surface["Ending surface"] == 3

    def test_cardinal_points_returns_correct_result(self, simple_system, expected_data):
        result = cardinal_points(simple_system, 2, 3)

        assert result.Data.General.Wavelength == expected_data.Data.General.Wavelength
        assert result.Data.General.Orientation == expected_data.Data.General.Orientation
        assert all(result.Data["Object Space"] == expected_data.Data["Object Space"])
        assert all(result.Data["Image Space"] == expected_data.Data["Image Space"])

    def test_cardinal_points_matches_reference_data(self, simple_system, reference_data):
        result = cardinal_points(simple_system, 2, 3)

        assert result.Data.General.Wavelength == reference_data.Data.General.Wavelength
        assert result.Data.General.Orientation == reference_data.Data.General.Orientation
        assert all(result.Data["Object Space"] == reference_data.Data["Object Space"])
        assert all(result.Data["Image Space"] == reference_data.Data["Image Space"])



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "DataType"

class DataType:
    Wavefront: DataType = None
    SpotRadius: DataType = None
    Spot_X: DataType = None
    Spot_Y: DataType = None
    StrehlRation: DataType = None
    PTWavefront: DataType = None



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations
from . import Aberrations, EncircledEnergy, ExtendedScene, Fans, Mtf, NSCSurface, Psf, RMS, Spot, Wavefront

__all__ = (
    "Aberrations",
    "EncircledEnergy",
    "ExtendedScene",
    "Fans",
    "Mtf",
    "NSCSurface",
    "Psf",
    "RMS",
    "Spot",
    "Wavefront",
    "AxisType",
    "DetectorViewerScaleTypes",
    "DisplayOption",
    "HuygensPsfTypes",
    "Parity",
    "Polarizations",
    "PsfSpread",
    "PsfTypes",
    "ReferenceGia",
    "Rotations",
    "ScanTypes",
    "SourceGia",
    "STAREffectsOptions",
)

class AxisType:
    X: AxisType = None
    Y: AxisType = None
    Z: AxisType = None

class DetectorViewerScaleTypes:
    Linear: DetectorViewerScaleTypes = None
    Log_Minus_5: DetectorViewerScaleTypes = None
    Normalized: DetectorViewerScaleTypes = None
    Log_Minus_10: DetectorViewerScaleTypes = None
    Log_Minus_15: DetectorViewerScaleTypes = None

class DisplayOption:
    AllRays: DisplayOption = None
    FailedRays: DisplayOption = None
    PassedRays: DisplayOption = None

class HuygensPsfTypes:
    Linear: HuygensPsfTypes = None
    Log_Minus_1: HuygensPsfTypes = None
    Log_Minus_2: HuygensPsfTypes = None
    Log_Minus_3: HuygensPsfTypes = None
    Log_Minus_4: HuygensPsfTypes = None
    Log_Minus_5: HuygensPsfTypes = None
    Real: HuygensPsfTypes = None
    Imaginary: HuygensPsfTypes = None
    Phase: HuygensPsfTypes = None

class Parity:
    Even: Parity = None
    Odd: Parity = None

class Polarizations:
    None_: Polarizations = None
    Ex: Polarizations = None
    Ey: Polarizations = None
    Ez: Polarizations = None

class PsfSpread:
    Line: PsfSpread = None
    Edge: PsfSpread = None

class PsfTypes:
    X_Linear: PsfTypes = None
    Y_Linear: PsfTypes = None
    X_Logarithmic: PsfTypes = None
    Y_Logarithmic: PsfTypes = None
    X_Phase: PsfTypes = None
    Y_Phase: PsfTypes = None
    X_RealPart: PsfTypes = None
    Y_RealPart: PsfTypes = None
    X_ImaginaryPart: PsfTypes = None
    Y_ImaginaryPart: PsfTypes = None

class ReferenceGia:
    ChiefRay: ReferenceGia = None
    Vertex: ReferenceGia = None
    PrimaryChief: ReferenceGia = None
    Centroid: ReferenceGia = None

class Rotations:
    Rotate_0: Rotations = None
    Rotate_90: Rotations = None
    Rotate_180: Rotations = None
    Rotate_270: Rotations = None

class ScanTypes:
    Plus_Y: ScanTypes = None
    Plus_X: ScanTypes = None
    Minus_Y: ScanTypes = None
    Minus_X: ScanTypes = None

class SourceGia:
    Uniform: SourceGia = None
    Lambertian: SourceGia = None

class STAREffectsOptions:
    On: STAREffectsOptions = None
    Difference: STAREffectsOptions = None



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations
from . import PhysicalOptics, RayTracing, Settings, Tolerancing

__all__ = (
    "PhysicalOptics",
    "RayTracing",
    "Settings",
    "Tolerancing",
    "AnalysisIDM",
    "Beam",
    "BestFitSphereOptions",
    "ColorPaletteType",
    "DetectorViewerShowAsTypes",
    "DetectorViewerShowDataTypes",
    "ErrorType",
    "GiaShowAsTypes",
    "GridPlotType",
    "HuygensShowAsTypes",
    "HuygensSurfaceMftShowAsTypes",
    "POPSampling",
    "RemoveOptions",
    "SampleSizes",
    "SampleSizes_ContrastLoss",
    "SampleSizes_Pow2Plus1",
    "SampleSizes_Pow2Plus1_X",
    "ShowAs",
    "SurfaceCurvatureCrossData",
    "SurfaceCurvatureData",
    "SurfacePhaseData",
    "SurfacePhaseSlopeCrossData",
    "SurfacePhaseSlopeData",
    "SurfaceSagData",
    "SurfaceSlopeCrossData",
    "SurfaceSlopeData",
    "UserAnalysisDataType",
)

class AnalysisIDM:
    RayFan: AnalysisIDM = None
    OpticalPathFan: AnalysisIDM = None
    PupilAberrationFan: AnalysisIDM = None
    FieldCurvatureAndDistortion: AnalysisIDM = None
    FocalShiftDiagram: AnalysisIDM = None
    GridDistortion: AnalysisIDM = None
    LateralColor: AnalysisIDM = None
    LongitudinalAberration: AnalysisIDM = None
    RayTrace: AnalysisIDM = None
    SeidelCoefficients: AnalysisIDM = None
    SeidelDiagram: AnalysisIDM = None
    ZernikeAnnularCoefficients: AnalysisIDM = None
    ZernikeCoefficientsVsField: AnalysisIDM = None
    ZernikeFringeCoefficients: AnalysisIDM = None
    ZernikeStandardCoefficients: AnalysisIDM = None
    FftMtf: AnalysisIDM = None
    FftThroughFocusMtf: AnalysisIDM = None
    GeometricThroughFocusMtf: AnalysisIDM = None
    GeometricMtf: AnalysisIDM = None
    FftMtfMap: AnalysisIDM = None
    GeometricMtfMap: AnalysisIDM = None
    FftSurfaceMtf: AnalysisIDM = None
    FftMtfvsField: AnalysisIDM = None
    GeometricMtfvsField: AnalysisIDM = None
    HuygensMtfvsField: AnalysisIDM = None
    HuygensMtf: AnalysisIDM = None
    HuygensSurfaceMtf: AnalysisIDM = None
    HuygensThroughFocusMtf: AnalysisIDM = None
    FftPsf: AnalysisIDM = None
    FftPsfCrossSection: AnalysisIDM = None
    FftPsfLineEdgeSpread: AnalysisIDM = None
    HuygensPsfCrossSection: AnalysisIDM = None
    HuygensPsf: AnalysisIDM = None
    DiffractionEncircledEnergy: AnalysisIDM = None
    GeometricEncircledEnergy: AnalysisIDM = None
    GeometricLineEdgeSpread: AnalysisIDM = None
    ExtendedSourceEncircledEnergy: AnalysisIDM = None
    SurfaceCurvatureCross: AnalysisIDM = None
    SurfacePhaseCross: AnalysisIDM = None
    SurfaceSagCross: AnalysisIDM = None
    SurfaceCurvature: AnalysisIDM = None
    SurfacePhase: AnalysisIDM = None
    SurfaceSag: AnalysisIDM = None
    StandardSpot: AnalysisIDM = None
    ThroughFocusSpot: AnalysisIDM = None
    FullFieldSpot: AnalysisIDM = None
    MatrixSpot: AnalysisIDM = None
    ConfigurationMatrixSpot: AnalysisIDM = None
    RMSField: AnalysisIDM = None
    RMSFieldMap: AnalysisIDM = None
    RMSLambdaDiagram: AnalysisIDM = None
    RMSFocus: AnalysisIDM = None
    Foucault: AnalysisIDM = None
    Interferogram: AnalysisIDM = None
    WavefrontMap: AnalysisIDM = None
    DetectorViewer: AnalysisIDM = None
    Draw2D: AnalysisIDM = None
    Draw3D: AnalysisIDM = None
    ImageSimulation: AnalysisIDM = None
    GeometricImageAnalysis: AnalysisIDM = None
    IMABIMFileViewer: AnalysisIDM = None
    GeometricBitmapImageAnalysis: AnalysisIDM = None
    BitmapFileViewer: AnalysisIDM = None
    LightSourceAnalysis: AnalysisIDM = None
    PartiallyCoherentImageAnalysis: AnalysisIDM = None
    ExtendedDiffractionImageAnalysis: AnalysisIDM = None
    BiocularFieldOfViewAnalysis: AnalysisIDM = None
    BiocularDipvergenceConvergence: AnalysisIDM = None
    RelativeIllumination: AnalysisIDM = None
    VignettingDiagramSettings: AnalysisIDM = None
    FootprintSettings: AnalysisIDM = None
    YYbarDiagram: AnalysisIDM = None
    PowerFieldMapSettings: AnalysisIDM = None
    PowerPupilMapSettings: AnalysisIDM = None
    IncidentAnglevsImageHeight: AnalysisIDM = None
    FiberCouplingSettings: AnalysisIDM = None
    YNIContributions: AnalysisIDM = None
    SagTable: AnalysisIDM = None
    CardinalPoints: AnalysisIDM = None
    DispersionDiagram: AnalysisIDM = None
    GlassMap: AnalysisIDM = None
    AthermalGlassMap: AnalysisIDM = None
    InternalTransmissionvsWavelength: AnalysisIDM = None
    DispersionvsWavelength: AnalysisIDM = None
    GrinProfile: AnalysisIDM = None
    GradiumProfile: AnalysisIDM = None
    UniversalPlot1D: AnalysisIDM = None
    UniversalPlot2D: AnalysisIDM = None
    PolarizationRayTrace: AnalysisIDM = None
    PolarizationPupilMap: AnalysisIDM = None
    Transmission: AnalysisIDM = None
    PhaseAberration: AnalysisIDM = None
    TransmissionFan: AnalysisIDM = None
    ParaxialGaussianBeam: AnalysisIDM = None
    SkewGaussianBeam: AnalysisIDM = None
    PhysicalOpticsPropagation: AnalysisIDM = None
    BeamFileViewer: AnalysisIDM = None
    ReflectionvsAngle: AnalysisIDM = None
    TransmissionvsAngle: AnalysisIDM = None
    AbsorptionvsAngle: AnalysisIDM = None
    DiattenuationvsAngle: AnalysisIDM = None
    PhasevsAngle: AnalysisIDM = None
    RetardancevsAngle: AnalysisIDM = None
    ReflectionvsWavelength: AnalysisIDM = None
    TransmissionvsWavelength: AnalysisIDM = None
    AbsorptionvsWavelength: AnalysisIDM = None
    DiattenuationvsWavelength: AnalysisIDM = None
    PhasevsWavelength: AnalysisIDM = None
    RetardancevsWavelength: AnalysisIDM = None
    DirectivityPlot: AnalysisIDM = None
    SourcePolarViewer: AnalysisIDM = None
    PhotoluminscenceViewer: AnalysisIDM = None
    SourceSpectrumViewer: AnalysisIDM = None
    RadiantSourceModelViewerSettings: AnalysisIDM = None
    SurfaceDataSettings: AnalysisIDM = None
    PrescriptionDataSettings: AnalysisIDM = None
    FileComparatorSettings: AnalysisIDM = None
    PartViewer: AnalysisIDM = None
    ReverseRadianceAnalysis: AnalysisIDM = None
    PathAnalysis: AnalysisIDM = None
    FluxvsWavelength: AnalysisIDM = None
    RoadwayLighting: AnalysisIDM = None
    SourceIlluminationMap: AnalysisIDM = None
    ScatterFunctionViewer: AnalysisIDM = None
    ScatterPolarPlotSettings: AnalysisIDM = None
    ZemaxElementDrawing: AnalysisIDM = None
    ShadedModel: AnalysisIDM = None
    NSCShadedModel: AnalysisIDM = None
    NSC3DLayout: AnalysisIDM = None
    NSCObjectViewer: AnalysisIDM = None
    RayDatabaseViewer: AnalysisIDM = None
    ISOElementDrawing: AnalysisIDM = None
    SystemData: AnalysisIDM = None
    TestPlateList: AnalysisIDM = None
    SourceColorChart1931: AnalysisIDM = None
    SourceColorChart1976: AnalysisIDM = None
    PrescriptionGraphic: AnalysisIDM = None
    CriticalRayTracer: AnalysisIDM = None
    ContrastLoss: AnalysisIDM = None
    CoatingListing: AnalysisIDM = None
    FullFieldAberration: AnalysisIDM = None
    SurfaceSlope: AnalysisIDM = None
    SurfaceSlopeCross: AnalysisIDM = None
    QuickYield: AnalysisIDM = None
    SystemCheck: AnalysisIDM = None
    ToleranceYield: AnalysisIDM = None
    ToleranceHistogram: AnalysisIDM = None
    DiffEfficiency2D: AnalysisIDM = None
    DiffEfficiencyAngular: AnalysisIDM = None
    DiffEfficiencyChromatic: AnalysisIDM = None
    NSCSurfaceSag: AnalysisIDM = None
    NSCSingleRayTrace: AnalysisIDM = None
    NSCGeometricMtf: AnalysisIDM = None
    SurfacePhaseSlope: AnalysisIDM = None
    SurfacePhaseSlopeCross: AnalysisIDM = None
    STARAlignCheck: AnalysisIDM = None
    STARSysViewer: AnalysisIDM = None
    STAR2DDefPlot: AnalysisIDM = None
    STARPerfChange: AnalysisIDM = None
    STARIndexVsTemp: AnalysisIDM = None
    STARInspectFEA: AnalysisIDM = None
    UserDefinedCOM: AnalysisIDM = None
    XXXTemplateXXX: AnalysisIDM = None

class Beam:
    Reference: Beam = None
    Configuration_1: Beam = None
    Configuration_2: Beam = None
    Configuration_3: Beam = None
    Configuration_4: Beam = None
    Configuration_5: Beam = None
    Configuration_6: Beam = None
    Configuration_7: Beam = None
    Configuration_8: Beam = None
    Configuration_9: Beam = None
    Configuration_10: Beam = None
    Configuration_11: Beam = None
    Configuration_12: Beam = None
    Configuration_13: Beam = None
    Configuration_14: Beam = None
    Configuration_15: Beam = None
    Configuration_16: Beam = None
    Configuration_17: Beam = None
    Configuration_18: Beam = None
    Configuration_19: Beam = None
    Configuration_20: Beam = None

class BestFitSphereOptions:
    MinimumVolume: BestFitSphereOptions = None
    MinimumRMS: BestFitSphereOptions = None
    MinimumRMSWithOffset: BestFitSphereOptions = None

class ColorPaletteType:
    GreyScale: ColorPaletteType = None
    FalseColor: ColorPaletteType = None
    FalseColorOriginal: ColorPaletteType = None
    Viridis: ColorPaletteType = None
    Magma: ColorPaletteType = None

class DetectorViewerShowAsTypes:
    GreyScale: DetectorViewerShowAsTypes = None
    FullListing: DetectorViewerShowAsTypes = None
    Text_CrossSection_Row: DetectorViewerShowAsTypes = None
    AzimuthCrossSection: DetectorViewerShowAsTypes = None
    GreyScale_Inverted: DetectorViewerShowAsTypes = None
    Text_CrossSection_Column: DetectorViewerShowAsTypes = None
    FalseColor: DetectorViewerShowAsTypes = None
    FluxVsWaveLength: DetectorViewerShowAsTypes = None
    FalseColor_Inverted: DetectorViewerShowAsTypes = None
    TrueColor: DetectorViewerShowAsTypes = None
    CrossSection_Row: DetectorViewerShowAsTypes = None
    CrossSection: DetectorViewerShowAsTypes = None
    Color_CrossSection_Row: DetectorViewerShowAsTypes = None
    CrossSection_Column: DetectorViewerShowAsTypes = None
    Directivity_Full: DetectorViewerShowAsTypes = None
    Color_CrossSection_Column: DetectorViewerShowAsTypes = None
    GeometricMtf: DetectorViewerShowAsTypes = None
    Color_FluxVsWavelength: DetectorViewerShowAsTypes = None
    Directivity_Half: DetectorViewerShowAsTypes = None

class DetectorViewerShowDataTypes:
    IncoherentIrradiance: DetectorViewerShowDataTypes = None
    Polar_AngleSpace: DetectorViewerShowDataTypes = None
    IncoherentFluence: DetectorViewerShowDataTypes = None
    IncidentFlux: DetectorViewerShowDataTypes = None
    PositionSpace: DetectorViewerShowDataTypes = None
    IncoherentIlluminance: DetectorViewerShowDataTypes = None
    CoherentFluence: DetectorViewerShowDataTypes = None
    AngleSpace: DetectorViewerShowDataTypes = None
    CoherentIrradiance: DetectorViewerShowDataTypes = None
    AbsorbedFlux: DetectorViewerShowDataTypes = None
    CoherentIlluminance: DetectorViewerShowDataTypes = None
    AbsorbedFluxVolume: DetectorViewerShowDataTypes = None
    CoherentPhase: DetectorViewerShowDataTypes = None
    RadiantIntensity: DetectorViewerShowDataTypes = None
    LuminousIntensity: DetectorViewerShowDataTypes = None
    LuminancePositionSpace: DetectorViewerShowDataTypes = None
    RadiancePositionSpace: DetectorViewerShowDataTypes = None
    LuminanceAngleSpace: DetectorViewerShowDataTypes = None
    RadianceAngleSpace: DetectorViewerShowDataTypes = None

class ErrorType:
    Success: ErrorType = None
    InvalidParameter: ErrorType = None
    InvalidSettings: ErrorType = None
    Failed: ErrorType = None
    AnalysisUnavailableForProgramMode: ErrorType = None
    NotYetImplemented: ErrorType = None
    NoSolverLicenseAvailable: ErrorType = None
    ToolAlreadyOpen: ErrorType = None
    SequentialOnly: ErrorType = None
    NonSequentialOnly: ErrorType = None
    SingleNSCRayTraceSupported: ErrorType = None
    HPCNotAvailable: ErrorType = None
    FeatureNotSupported: ErrorType = None
    NotAvailableInLegacy: ErrorType = None

class GiaShowAsTypes:
    Surface: GiaShowAsTypes = None
    Contour: GiaShowAsTypes = None
    GreyScale: GiaShowAsTypes = None
    InverseGreyScale: GiaShowAsTypes = None
    FalseColor: GiaShowAsTypes = None
    InverseFalseColor: GiaShowAsTypes = None
    SpotDiagram: GiaShowAsTypes = None
    CrossX: GiaShowAsTypes = None
    CrossY: GiaShowAsTypes = None

class GridPlotType:
    Surface: GridPlotType = None
    Contour: GridPlotType = None
    GrayScale: GridPlotType = None
    InverseGrayScale: GridPlotType = None
    FalseColor: GridPlotType = None
    InverseFalseColor: GridPlotType = None

class HuygensShowAsTypes:
    Surface: HuygensShowAsTypes = None
    Contour: HuygensShowAsTypes = None
    GreyScale: HuygensShowAsTypes = None
    InverseGreyScale: HuygensShowAsTypes = None
    FalseColor: HuygensShowAsTypes = None
    InverseFalseColor: HuygensShowAsTypes = None
    TrueColor: HuygensShowAsTypes = None

class HuygensSurfaceMftShowAsTypes:
    GreyScale: HuygensSurfaceMftShowAsTypes = None
    InverseGreyScale: HuygensSurfaceMftShowAsTypes = None
    FalseColor: HuygensSurfaceMftShowAsTypes = None
    InverseFalseColor: HuygensSurfaceMftShowAsTypes = None

class POPSampling:
    S_32: POPSampling = None
    S_64: POPSampling = None
    S_128: POPSampling = None
    S_256: POPSampling = None
    S_512: POPSampling = None
    S_1024: POPSampling = None
    S_1K: POPSampling = None
    S_2048: POPSampling = None
    S_2K: POPSampling = None
    S_4K: POPSampling = None
    S_4096: POPSampling = None
    S_8192: POPSampling = None
    S_8K: POPSampling = None
    S_16384: POPSampling = None
    S_16K: POPSampling = None
    S_32K: POPSampling = None
    S_32768: POPSampling = None
    S_65536: POPSampling = None
    S_64K: POPSampling = None
    S_131072: POPSampling = None
    S_128K: POPSampling = None
    S_256K: POPSampling = None
    S_262144: POPSampling = None
    S_524288: POPSampling = None
    S_512K: POPSampling = None
    S_1048576: POPSampling = None
    S_1M: POPSampling = None
    S_2M: POPSampling = None
    S_2097152: POPSampling = None
    S_4194304: POPSampling = None
    S_4M: POPSampling = None
    S_8388608: POPSampling = None
    S_8M: POPSampling = None
    S_16M: POPSampling = None
    S_16777216: POPSampling = None
    S_33554432: POPSampling = None
    S_32M: POPSampling = None
    S_67108864: POPSampling = None
    S_64M: POPSampling = None
    S_128M: POPSampling = None
    S_134217728: POPSampling = None
    S_268435456: POPSampling = None
    S_256M: POPSampling = None
    S_536870912: POPSampling = None
    S_512M: POPSampling = None
    S_1073741824: POPSampling = None
    S_1G: POPSampling = None

class RemoveOptions:
    None_: RemoveOptions = None
    BaseROC: RemoveOptions = None
    BestFitSphere: RemoveOptions = None
    BaseSag: RemoveOptions = None
    CompositeSag: RemoveOptions = None

class SampleSizes:
    S_32x32: SampleSizes = None
    S_64x64: SampleSizes = None
    S_128x128: SampleSizes = None
    S_256x256: SampleSizes = None
    S_512x512: SampleSizes = None
    S_1024x1024: SampleSizes = None
    S_2048x2048: SampleSizes = None
    S_4096x4096: SampleSizes = None
    S_8192x8192: SampleSizes = None
    S_16384x16384: SampleSizes = None

class SampleSizes_ContrastLoss:
    S_3x3: SampleSizes_ContrastLoss = None
    S_5x5: SampleSizes_ContrastLoss = None
    S_7x7: SampleSizes_ContrastLoss = None
    S_9x9: SampleSizes_ContrastLoss = None
    S_11x11: SampleSizes_ContrastLoss = None
    S_13x13: SampleSizes_ContrastLoss = None
    S_15x15: SampleSizes_ContrastLoss = None
    S_17x17: SampleSizes_ContrastLoss = None
    S_19x19: SampleSizes_ContrastLoss = None
    S_21x21: SampleSizes_ContrastLoss = None
    S_23x23: SampleSizes_ContrastLoss = None
    S_25x25: SampleSizes_ContrastLoss = None
    S_27x27: SampleSizes_ContrastLoss = None
    S_29x29: SampleSizes_ContrastLoss = None
    S_31x31: SampleSizes_ContrastLoss = None
    S_33x33: SampleSizes_ContrastLoss = None
    S_35x35: SampleSizes_ContrastLoss = None
    S_37x37: SampleSizes_ContrastLoss = None
    S_39x39: SampleSizes_ContrastLoss = None
    S_41x41: SampleSizes_ContrastLoss = None
    S_43x43: SampleSizes_ContrastLoss = None
    S_45x45: SampleSizes_ContrastLoss = None
    S_47x47: SampleSizes_ContrastLoss = None
    S_49x49: SampleSizes_ContrastLoss = None
    S_51x51: SampleSizes_ContrastLoss = None
    S_53x53: SampleSizes_ContrastLoss = None
    S_55x55: SampleSizes_ContrastLoss = None
    S_57x57: SampleSizes_ContrastLoss = None
    S_59x59: SampleSizes_ContrastLoss = None
    S_61x61: SampleSizes_ContrastLoss = None
    S_63x63: SampleSizes_ContrastLoss = None
    S_65x65: SampleSizes_ContrastLoss = None
    S_67x67: SampleSizes_ContrastLoss = None
    S_69x69: SampleSizes_ContrastLoss = None
    S_71x71: SampleSizes_ContrastLoss = None
    S_73x73: SampleSizes_ContrastLoss = None
    S_75x75: SampleSizes_ContrastLoss = None
    S_77x77: SampleSizes_ContrastLoss = None
    S_79x79: SampleSizes_ContrastLoss = None
    S_81x81: SampleSizes_ContrastLoss = None
    S_83x83: SampleSizes_ContrastLoss = None

class SampleSizes_Pow2Plus1:
    S_33: SampleSizes_Pow2Plus1 = None
    S_65: SampleSizes_Pow2Plus1 = None
    S_129: SampleSizes_Pow2Plus1 = None
    S_257: SampleSizes_Pow2Plus1 = None
    S_513: SampleSizes_Pow2Plus1 = None
    S_1025: SampleSizes_Pow2Plus1 = None
    S_2049: SampleSizes_Pow2Plus1 = None
    S_4097: SampleSizes_Pow2Plus1 = None
    S_8193: SampleSizes_Pow2Plus1 = None

class SampleSizes_Pow2Plus1_X:
    S_33x33: SampleSizes_Pow2Plus1_X = None
    S_65x65: SampleSizes_Pow2Plus1_X = None
    S_129x129: SampleSizes_Pow2Plus1_X = None
    S_257x257: SampleSizes_Pow2Plus1_X = None
    S_513x513: SampleSizes_Pow2Plus1_X = None
    S_1025x1025: SampleSizes_Pow2Plus1_X = None
    S_2049x2049: SampleSizes_Pow2Plus1_X = None
    S_4097x4097: SampleSizes_Pow2Plus1_X = None
    S_8193x8193: SampleSizes_Pow2Plus1_X = None

class ShowAs:
    Surface: ShowAs = None
    Contour: ShowAs = None
    GreyScale: ShowAs = None
    InverseGreyScale: ShowAs = None
    FalseColor: ShowAs = None
    InverseFalseColor: ShowAs = None

class SurfaceCurvatureCrossData:
    TangentialCurvature: SurfaceCurvatureCrossData = None
    SagittalCurvature: SurfaceCurvatureCrossData = None
    X_Curvature: SurfaceCurvatureCrossData = None
    Y_Curvature: SurfaceCurvatureCrossData = None
    CurvatureModulus: SurfaceCurvatureCrossData = None
    CurvatureUnused: SurfaceCurvatureCrossData = None

class SurfaceCurvatureData:
    TangentialCurvature: SurfaceCurvatureData = None
    SagittalCurvature: SurfaceCurvatureData = None
    X_Curvature: SurfaceCurvatureData = None
    Y_Curvature: SurfaceCurvatureData = None
    CurvatureModulus: SurfaceCurvatureData = None
    CurvatureUnused: SurfaceCurvatureData = None

class SurfacePhaseData:
    SurfacePhase: SurfacePhaseData = None

class SurfacePhaseSlopeCrossData:
    PhaseSlopeTangential: SurfacePhaseSlopeCrossData = None
    PhaseSlopeSagittal: SurfacePhaseSlopeCrossData = None
    PhaseSlopeX: SurfacePhaseSlopeCrossData = None
    PhaseSlopeY: SurfacePhaseSlopeCrossData = None
    PhaseSlopeModulus: SurfacePhaseSlopeCrossData = None
    PhaseSlopeUnused: SurfacePhaseSlopeCrossData = None

class SurfacePhaseSlopeData:
    PhaseSlopeTangential: SurfacePhaseSlopeData = None
    PhaseSlopeSagittal: SurfacePhaseSlopeData = None
    PhaseSlopeX: SurfacePhaseSlopeData = None
    PhaseSlopeY: SurfacePhaseSlopeData = None
    PhaseSlopeModulus: SurfacePhaseSlopeData = None
    PhaseSlopeUnused: SurfacePhaseSlopeData = None

class SurfaceSagData:
    SurfaceSag: SurfaceSagData = None

class SurfaceSlopeCrossData:
    TangentialSlope: SurfaceSlopeCrossData = None
    SagittalSlope: SurfaceSlopeCrossData = None
    XSlope: SurfaceSlopeCrossData = None
    YSlope: SurfaceSlopeCrossData = None
    SlopeModulus: SurfaceSlopeCrossData = None
    SlopeUnused: SurfaceSlopeCrossData = None

class SurfaceSlopeData:
    SurfaceSlope: SurfaceSlopeData = None
    TangentialSlope: SurfaceSlopeData = None
    SagittalSlope: SurfaceSlopeData = None
    XSlope: SurfaceSlopeData = None
    YSlope: SurfaceSlopeData = None
    SlopeModulus: SurfaceSlopeData = None
    SlopeUnused: SurfaceSlopeData = None

class UserAnalysisDataType:
    None_: UserAnalysisDataType = None
    Line2D: UserAnalysisDataType = None
    Grid: UserAnalysisDataType = None
    GridRGB: UserAnalysisDataType = None
    Text: UserAnalysisDataType = None



import numpy as np
import pytest

from zospy.analyses.old.extendedscene import geometric_image_analysis

pytestmark = pytest.mark.old_analyses


class TestGeometricImageAnalysis:
    def test_can_run_geometric_image_analysis(self, simple_system):
        result = geometric_image_analysis(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = geometric_image_analysis(simple_system)

        assert result.from_json(result.to_json())

    @pytest.mark.parametrize(
        "show_as,field_size,total_watts,rays_x_1000",
        [
            ("Surface", 0, 1, 100000),
            ("Surface", 10, 1, 100000),
            ("Surface", 10, 100, 100000),
            ("Surface", 0, 100, 100000),
            ("CrossX", 0, 1, 100000),
            ("CrossX", 10, 1, 100000),
            ("CrossX", 10, 100, 100000),
            ("CrossX", 0, 100, 100000),
        ],
    )
    def test_geometric_image_analysis_returns_correct_result(
        self, simple_system, show_as, field_size, total_watts, rays_x_1000, expected_data
    ):
        result = geometric_image_analysis(
            simple_system, show_as=show_as, field_size=field_size, total_watts=total_watts, rays_x_1000=rays_x_1000
        )

        # Data in result.Data
        # rtol lower than normal due to randomness in calculated rays
        assert np.allclose(
            result.Data.to_numpy(dtype=float), expected_data.Data.to_numpy(dtype=float), equal_nan=True, rtol=1e-2
        )

    @pytest.mark.parametrize(
        "show_as,field_size,total_watts,rays_x_1000",
        [
            ("Surface", 0, 1, 100000),
            ("Surface", 10, 1, 100000),
            ("Surface", 10, 100, 100000),
            ("Surface", 0, 100, 100000),
            ("CrossX", 0, 1, 100000),
            ("CrossX", 10, 1, 100000),
            ("CrossX", 10, 100, 100000),
            ("CrossX", 0, 100, 100000),
        ],
    )
    def test_geometric_image_analysis_matches_reference_data(
        self, simple_system, show_as, field_size, total_watts, rays_x_1000, reference_data
    ):
        result = geometric_image_analysis(
            simple_system, show_as=show_as, field_size=field_size, total_watts=total_watts, rays_x_1000=rays_x_1000
        )

        # Data in result.Data
        # rtol lower than normal due to randomness in calculated rays
        assert np.allclose(
            result.Data.to_numpy(dtype=float), reference_data.Data.to_numpy(dtype=float), equal_nan=True, rtol=1e-2
        )



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis import SampleSizes
from zospy.api._ZOSAPI.Analysis.Settings import IAS_

__all__ = ("DataType", "FoucaultShowAs", "IAS_Foucault", "KnifeType", "Types")

class DataType:
    Computed = 0
    Reference = 1
    Difference = 2

class FoucaultShowAs:
    Surface = 0
    Contour = 1
    GreyScale = 2
    InverseGreyScale = 3
    FalseColor = 4
    InverseFalseColor = 5
    X_CrossSection = 6
    Y_CrossSection = 7

class IAS_Foucault(IAS_):
    @property
    def Field(self) -> IAS_Field: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def Type(self) -> Types: ...
    @Type.setter
    def Type(self, value: Types) -> None: ...
    @property
    def Sampling(self) -> SampleSizes: ...
    @Sampling.setter
    def Sampling(self, value: SampleSizes) -> None: ...
    @property
    def ShowAs(self) -> FoucaultShowAs: ...
    @ShowAs.setter
    def ShowAs(self, value: FoucaultShowAs) -> None: ...
    @property
    def Knife(self) -> KnifeType: ...
    @Knife.setter
    def Knife(self, value: KnifeType) -> None: ...
    @property
    def Data(self) -> DataType: ...
    @Data.setter
    def Data(self, value: DataType) -> None: ...
    @property
    def RowColumn(self) -> int: ...
    @RowColumn.setter
    def RowColumn(self, value: int) -> None: ...
    @property
    def UsePolarization(self) -> bool: ...
    @UsePolarization.setter
    def UsePolarization(self, value: bool) -> None: ...
    @property
    def Y_Position(self) -> float: ...
    @Y_Position.setter
    def Y_Position(self, value: float) -> None: ...
    @property
    def Decenter_X(self) -> float: ...
    @Decenter_X.setter
    def Decenter_X(self, value: float) -> None: ...
    @property
    def Decenter_Y(self) -> float: ...
    @Decenter_Y.setter
    def Decenter_Y(self, value: float) -> None: ...
    @property
    def Scale_X(self) -> float: ...
    @Scale_X.setter
    def Scale_X(self, value: float) -> None: ...
    @property
    def Scale_Y(self) -> float: ...
    @Scale_Y.setter
    def Scale_Y(self, value: float) -> None: ...
    @property
    def Source(self) -> str: ...
    @Source.setter
    def Source(self, value: str) -> None: ...
    @property
    def SaveBMP(self) -> str: ...
    @SaveBMP.setter
    def SaveBMP(self, value: str) -> None: ...

class KnifeType:
    Horiz_Above = 0
    Horiz_Below = 1
    Vert_Right = 2
    Vert_Left = 3

class Types:
    Linear = 0
    Log_Minus_3 = 1
    Log_Minus_6 = 2
    Log_Minus_9 = 3
    Log_Minus_12 = 4
    Log_Minus_15 = 5
    Log_Minus_18 = 6



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("ConvertAsphereTypes", "FindBestAsphereTypes")

class ConvertAsphereTypes:
    EvenAsphere: ConvertAsphereTypes = None
    ExtendedAsphere: ConvertAsphereTypes = None
    QTypeAsphereType0: ConvertAsphereTypes = None
    QTypeAsphereType1: ConvertAsphereTypes = None

class FindBestAsphereTypes:
    Conic: FindBestAsphereTypes = None
    Polynomial_4thOrder: FindBestAsphereTypes = None
    Polynomial_6thOrder: FindBestAsphereTypes = None
    Polynomial_8thOrder: FindBestAsphereTypes = None
    Polynomial_10thOrder: FindBestAsphereTypes = None
    Polynomial_12thOrder: FindBestAsphereTypes = None



import numpy as np
import pytest

from zospy.analyses.old.physicaloptics import (
    physical_optics_propagation,
    pop_create_beam_parameter_dict,
    pop_create_fiber_parameter_dict,
)

pytestmark = pytest.mark.old_analyses


class TestPhysicalOpticsPropagation:
    def test_can_run_physical_optics_propagation(self, simple_system):
        result = physical_optics_propagation(simple_system)

        assert result.Data is not None

    def test_to_json(self, simple_system):
        result = physical_optics_propagation(simple_system)

        assert result.from_json(result.to_json())

    def test_can_create_beam_parameter_dict(self, simple_system):
        param_dict = pop_create_beam_parameter_dict(simple_system)

        assert isinstance(param_dict, dict)

    @pytest.mark.parametrize(
        "beam_type,expected_parameters",
        [
            (
                "GaussianWaist",
                {"Waist X", "Waist Y", "Decenter X", "Decenter Y", "Aperture X", "Aperture Y", "Order X", "Order Y"},
            ),
            ("GaussianSizeAngle", {"Size X", "Size Y", "Angle X (deg)", "Angle Y (deg)", "Decenter X:", "Decenter Y:"}),
            ("TopHat", {"Waist X", "Waist Y", "Decenter X", "Decenter Y"}),
            (
                "AstigmaticGaussian",
                {"Waist X", "Waist Y", "Waist Position X", "Waist Position Y", "Decenter X:", "Decenter Y:"},
            ),
            # TODO: create bug report regarding colons in decenter x and y
        ],
    )
    def test_create_beam_parameter_dict_returns_correct_parameters(self, simple_system, beam_type, expected_parameters):
        parameter_dict = pop_create_beam_parameter_dict(simple_system, beam_type=beam_type)

        assert set(parameter_dict.keys()) == expected_parameters

    def test_can_create_fiber_parameter_dict(self, simple_system):
        param_dict = pop_create_fiber_parameter_dict(simple_system)

        assert isinstance(param_dict, dict)

    @pytest.mark.parametrize(
        "fiber_type,expected_parameters",
        [
            (
                "GaussianWaist",
                {"Waist X", "Waist Y", "Decenter X", "Decenter Y", "Aperture X", "Aperture Y", "Order X", "Order Y"},
            ),
            ("GaussianSizeAngle", {"Size X", "Size Y", "Angle X (deg)", "Angle Y (deg)", "Decenter X:", "Decenter Y:"}),
            ("TopHat", {"Waist X", "Waist Y", "Decenter X", "Decenter Y"}),
            (
                "AstigmaticGaussian",
                {"Waist X", "Waist Y", "Waist Position X", "Waist Position Y", "Decenter X:", "Decenter Y:"},
            ),
            # TODO: create bug report regarding colons in decenter x and y
        ],
    )
    def test_create_fiber_parameter_dict_returns_correct_parameters(
        self, simple_system, fiber_type, expected_parameters
    ):
        parameter_dict = pop_create_fiber_parameter_dict(simple_system, fiber_type=fiber_type)

        assert set(parameter_dict.keys()) == expected_parameters

    @pytest.mark.parametrize(
        "compute_fiber_coupling_integral,fiber_type,fiber_parameters",
        [
            (False, "TopHat", None),
            (True, "TopHat", None),
            (
                True,
                "GaussianWaist",
                {
                    "Waist X": 0.03,
                    "Waist Y": 0.03,
                    "Decenter X": 0.0,
                    "Decenter Y": 0.0,
                    "Aperture X": 0.0,
                    "Aperture Y": 0.0,
                    "Order X": 0.0,
                    "Order Y": 0.0,
                },
            ),
        ],
    )
    @pytest.mark.parametrize(
        "beam_type,beam_parameters,auto_calculate_beam_sampling",
        [
            ("GaussianWaist", None, False),
            (
                "GaussianWaist",
                {
                    "Waist X": 1.5,
                    "Waist Y": 1.5,
                    "Decenter X": 0.1,
                    "Decenter Y": 0.1,
                    "Aperture X": 0.0,
                    "Aperture Y": 0.0,
                    "Order X": 0.0,
                    "Order Y": 0.0,
                },
                False,
            ),
            ("TopHat", {"Waist X": 0.5, "Waist Y": 0.5, "Decenter X": 0.0, "Decenter Y": 0.0}, False),
            ("TopHat", {"Waist X": 0.5, "Waist Y": 0.5, "Decenter X": 0.0, "Decenter Y": 0.0}, True),
        ],
    )
    @pytest.mark.parametrize(
        "surface_to_beam,data_type,use_total_power,use_peak_irradiance",
        [
            (0.0, "Irradiance", True, False),
            (1.0, "Irradiance", True, False),
            (0.0, "Irradiance", False, True),
            (0.0, "Phase", True, False),
        ],
    )
    def test_physical_optics_propagation_returns_correct_result(
        self,
        simple_system,
        surface_to_beam,
        beam_type,
        beam_parameters,
        use_total_power,
        use_peak_irradiance,
        data_type,
        compute_fiber_coupling_integral,
        fiber_type,
        fiber_parameters,
        auto_calculate_beam_sampling,
        expected_data,
    ):
        result = physical_optics_propagation(
            simple_system,
            surface_to_beam=surface_to_beam,
            beam_type=beam_type,
            beam_parameters=beam_parameters,
            use_total_power=use_total_power,
            use_peak_irradiance=use_peak_irradiance,
            data_type=data_type,
            compute_fiber_coupling_integral=compute_fiber_coupling_integral,
            fiber_type=fiber_type,
            fiber_parameters=fiber_parameters,
            auto_calculate_beam_sampling=auto_calculate_beam_sampling,
        )

        assert np.allclose(result.Data.astype(float), expected_data.Data.astype(float), rtol=1e-3)

    @pytest.mark.parametrize(
        "compute_fiber_coupling_integral,fiber_type,fiber_parameters",
        [
            (False, "TopHat", None),
            (True, "TopHat", None),
            (
                True,
                "GaussianWaist",
                {
                    "Waist X": 0.03,
                    "Waist Y": 0.03,
                    "Decenter X": 0.0,
                    "Decenter Y": 0.0,
                    "Aperture X": 0.0,
                    "Aperture Y": 0.0,
                    "Order X": 0.0,
                    "Order Y": 0.0,
                },
            ),
        ],
    )
    @pytest.mark.parametrize(
        "beam_type,beam_parameters,auto_calculate_beam_sampling",
        [
            ("GaussianWaist", None, False),
            (
                "GaussianWaist",
                {
                    "Waist X": 1.5,
                    "Waist Y": 1.5,
                    "Decenter X": 0.1,
                    "Decenter Y": 0.1,
                    "Aperture X": 0.0,
                    "Aperture Y": 0.0,
                    "Order X": 0.0,
                    "Order Y": 0.0,
                },
                False,
            ),
            ("TopHat", {"Waist X": 0.5, "Waist Y": 0.5, "Decenter X": 0.0, "Decenter Y": 0.0}, False),
            ("TopHat", {"Waist X": 0.5, "Waist Y": 0.5, "Decenter X": 0.0, "Decenter Y": 0.0}, True),
        ],
    )
    @pytest.mark.parametrize(
        "surface_to_beam,data_type,use_total_power,use_peak_irradiance",
        [
            (0.0, "Irradiance", True, False),
            (1.0, "Irradiance", True, False),
            (0.0, "Irradiance", False, True),
            (0.0, "Phase", True, False),
        ],
    )
    def test_physical_optics_propagation_matches_reference_data(
        self,
        simple_system,
        surface_to_beam,
        beam_type,
        beam_parameters,
        use_total_power,
        use_peak_irradiance,
        data_type,
        compute_fiber_coupling_integral,
        fiber_type,
        fiber_parameters,
        auto_calculate_beam_sampling,
        reference_data,
    ):
        result = physical_optics_propagation(
            simple_system,
            surface_to_beam=surface_to_beam,
            beam_type=beam_type,
            beam_parameters=beam_parameters,
            use_total_power=use_total_power,
            use_peak_irradiance=use_peak_irradiance,
            data_type=data_type,
            compute_fiber_coupling_integral=compute_fiber_coupling_integral,
            fiber_type=fiber_type,
            fiber_parameters=fiber_parameters,
            auto_calculate_beam_sampling=auto_calculate_beam_sampling,
        )

        assert np.allclose(result.Data.astype(float), reference_data.Data.astype(float), rtol=1e-3)

    @pytest.mark.parametrize(
        "use_total_power,use_peak_irradiance",
        [
            (True, True),
            (False, False),
        ],
    )
    def test_physical_optics_propagation_with_matching_use_total_power_and_peak_irradiance_raises_exception(
        self, simple_system, use_total_power, use_peak_irradiance
    ):
        with pytest.raises(
            ValueError,
            match="Either use_total_power or use_peak_irradiance should be True, they cannot both be True or False",
        ):
            physical_optics_propagation(
                simple_system, use_total_power=use_total_power, use_peak_irradiance=use_peak_irradiance
            )

    def test_physical_optics_propagation_with_wrong_beam_parameter_raises_exception(self, simple_system):
        with pytest.raises(ValueError, match="The following .+ parameters are specified but not accepted"):
            physical_optics_propagation(simple_system, beam_type="GaussianWaist", beam_parameters={"WrongName": 1})

    def test_physical_optics_propagation_with_wrong_fiber_parameter_raises_exception(self, simple_system):
        with pytest.raises(ValueError, match="The following .+ parameters are specified but not accepted"):
            physical_optics_propagation(
                simple_system,
                compute_fiber_coupling_integral=True,
                fiber_type="GaussianWaist",
                fiber_parameters={"WrongName": 1},
            )



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "MultiConfigOperandType"

class MultiConfigOperandType:
    MOFF: MultiConfigOperandType = None
    AICN: MultiConfigOperandType = None
    AFOC: MultiConfigOperandType = None
    APDF: MultiConfigOperandType = None
    APDT: MultiConfigOperandType = None
    APDX: MultiConfigOperandType = None
    APDY: MultiConfigOperandType = None
    APER: MultiConfigOperandType = None
    APMN: MultiConfigOperandType = None
    APMX: MultiConfigOperandType = None
    APTP: MultiConfigOperandType = None
    CADX: MultiConfigOperandType = None
    CADY: MultiConfigOperandType = None
    CAOR: MultiConfigOperandType = None
    CATX: MultiConfigOperandType = None
    CATY: MultiConfigOperandType = None
    CATZ: MultiConfigOperandType = None
    CBDX: MultiConfigOperandType = None
    CBDY: MultiConfigOperandType = None
    CBOR: MultiConfigOperandType = None
    CBTX: MultiConfigOperandType = None
    CBTY: MultiConfigOperandType = None
    CBTZ: MultiConfigOperandType = None
    CONN: MultiConfigOperandType = None
    COTN: MultiConfigOperandType = None
    CPCN: MultiConfigOperandType = None
    CROR: MultiConfigOperandType = None
    CRSR: MultiConfigOperandType = None
    CRVT: MultiConfigOperandType = None
    CSP1: MultiConfigOperandType = None
    CSP2: MultiConfigOperandType = None
    CWGT: MultiConfigOperandType = None
    EDVA: MultiConfigOperandType = None
    FLTP: MultiConfigOperandType = None
    FLWT: MultiConfigOperandType = None
    FVAN: MultiConfigOperandType = None
    FVCX: MultiConfigOperandType = None
    FVCY: MultiConfigOperandType = None
    FVDX: MultiConfigOperandType = None
    FVDY: MultiConfigOperandType = None
    GCRS: MultiConfigOperandType = None
    GLSS: MultiConfigOperandType = None
    GPEX: MultiConfigOperandType = None
    GPEY: MultiConfigOperandType = None
    GPIU: MultiConfigOperandType = None
    GPJX: MultiConfigOperandType = None
    GPJY: MultiConfigOperandType = None
    GPPX: MultiConfigOperandType = None
    GPPY: MultiConfigOperandType = None
    GQPO: MultiConfigOperandType = None
    HOLD: MultiConfigOperandType = None
    IGNM: MultiConfigOperandType = None
    IGNR: MultiConfigOperandType = None
    LTTL: MultiConfigOperandType = None
    MABB: MultiConfigOperandType = None
    MCOM: MultiConfigOperandType = None
    MDPG: MultiConfigOperandType = None
    MIND: MultiConfigOperandType = None
    MTFU: MultiConfigOperandType = None
    NCOM: MultiConfigOperandType = None
    NCOT: MultiConfigOperandType = None
    NGLS: MultiConfigOperandType = None
    NPAR: MultiConfigOperandType = None
    NPOS: MultiConfigOperandType = None
    NPRO: MultiConfigOperandType = None
    PAR1: MultiConfigOperandType = None
    PAR2: MultiConfigOperandType = None
    PAR3: MultiConfigOperandType = None
    PAR4: MultiConfigOperandType = None
    PAR5: MultiConfigOperandType = None
    PAR6: MultiConfigOperandType = None
    PAR7: MultiConfigOperandType = None
    PAR8: MultiConfigOperandType = None
    PRAM: MultiConfigOperandType = None
    PRES: MultiConfigOperandType = None
    PRWV: MultiConfigOperandType = None
    PSCX: MultiConfigOperandType = None
    PSCY: MultiConfigOperandType = None
    PSHX: MultiConfigOperandType = None
    PSHY: MultiConfigOperandType = None
    PSHZ: MultiConfigOperandType = None
    PSP1: MultiConfigOperandType = None
    PSP2: MultiConfigOperandType = None
    PSP3: MultiConfigOperandType = None
    PUCN: MultiConfigOperandType = None
    PXAR: MultiConfigOperandType = None
    RAAM: MultiConfigOperandType = None
    SATP: MultiConfigOperandType = None
    SDIA: MultiConfigOperandType = None
    SDRW: MultiConfigOperandType = None
    STPS: MultiConfigOperandType = None
    SWCN: MultiConfigOperandType = None
    TCEX: MultiConfigOperandType = None
    TELE: MultiConfigOperandType = None
    TEMP: MultiConfigOperandType = None
    THIC: MultiConfigOperandType = None
    TSP1: MultiConfigOperandType = None
    TSP2: MultiConfigOperandType = None
    TSP3: MultiConfigOperandType = None
    UDAF: MultiConfigOperandType = None
    WAVE: MultiConfigOperandType = None
    WLWT: MultiConfigOperandType = None
    XFIE: MultiConfigOperandType = None
    YFIE: MultiConfigOperandType = None
    OPDR: MultiConfigOperandType = None
    SRTS: MultiConfigOperandType = None
    MCHI: MultiConfigOperandType = None
    IGTO: MultiConfigOperandType = None
    FTAN: MultiConfigOperandType = None
    FCMM: MultiConfigOperandType = None
    CHZN: MultiConfigOperandType = None
    MCSD: MultiConfigOperandType = None
    CSDX: MultiConfigOperandType = None
    CSDY: MultiConfigOperandType = None
    CSTX: MultiConfigOperandType = None
    CSTY: MultiConfigOperandType = None
    CSTZ: MultiConfigOperandType = None



"""System Data analysis."""

from pydantic import Field

from zospy.analyses.base import BaseAnalysisWrapper
from zospy.analyses.decorators import analysis_result
from zospy.analyses.parsers import ZospyTransformer

__all__ = ("SystemData",)

from zospy.analyses.parsers.transformers import SimpleField


class SystemDataTransformer(ZospyTransformer):
    def general_lens_data(self, args):
        return SimpleField("General Lens Data", self.dict(args[1:]))

    def system_aperture(self, args):
        name, aperture_type, value = args

        return SimpleField(str(name), {"type": aperture_type, "value": value})

    def apodization(self, args):
        name, apodization_type, factor = args

        return SimpleField(str(name), {"type": apodization_type, "factor": factor})

    def efl_air(self, args):
        name, value = args

        return SimpleField("Effective Focal Length (air)", value)

    def efl_image(self, args):
        name, value = args

        return SimpleField("Effective Focal Length (image)", value)

    def field_data(self, args):
        return SimpleField("Fields", self.dict(args))

    def fields_table(self, args):
        header, rows = args[0]

        return SimpleField("Table", [dict(zip(header, row)) for row in rows])

    def vignetting_table(self, args):
        header, rows = args[0]

        return SimpleField("Vignetting Factors", [dict(zip(header, row)) for row in rows])

    def wavelength_data(self, args):
        return SimpleField("Wavelengths", self.dict(args))

    def wavelength_table(self, args):
        header, rows = args[0]

        return SimpleField("Table", [dict(zip(header, row)) for row in rows])

    def abcd_matrix(self, args):
        return SimpleField("Predicted coordinate ABCD matrix", self.dict(args))


@analysis_result
class SystemAperture:
    type: str
    value: float


@analysis_result
class Apodization:
    type: str
    factor: float


@analysis_result
class GeneralLensData:
    surfaces: int = Field(alias="Surfaces")
    stop: int = Field(alias="Stop")
    system_aperture: SystemAperture = Field(alias="System Aperture")
    fast_semi_diameters: bool = Field(alias="Fast Semi-Diameters")
    field_unpolarized: bool = Field(alias="Field Unpolarized")
    convert_thin_film_phase_to_ray_equivalent: bool = Field(alias="Convert thin film phase to ray equivalent")
    j_e_conversion_method: str = Field(alias="J/E Conversion Method")
    glass_catalogs: list[str] = Field(alias="Glass Catalogs")
    ray_aiming: bool = Field(alias="Ray Aiming")
    apodization: Apodization = Field(alias="Apodization")
    reference_opd: str = Field(alias="Reference OPD")
    paraxial_rays_setting: str = Field(alias="Paraxial Rays Setting")
    method_to_compute_f_number: str = Field(alias="Method to Compute F/#")
    method_to_compute_huygens_integral: str = Field(alias="Method to Compute Huygens Integral")
    print_coordinate_breaks: bool = Field(alias="Print Coordinate Breaks")
    multi_threading: bool = Field(alias="Multi-Threading")
    opd_modulo_2_pi: bool = Field(alias="OPD Modulo 2 Pi")
    temperature: float = Field(alias="Temperature (C)")
    pressure: float = Field(alias="Pressure (ATM)")
    adjust_index_data_to_environment: bool = Field(alias="Adjust Index Data To Environment")
    effective_focal_length_air: float = Field(alias="Effective Focal Length (air)")
    effective_focal_length_image: float = Field(alias="Effective Focal Length (image)")
    back_focal_length: float = Field(alias="Back Focal Length")
    total_track: float = Field(alias="Total Track")
    image_space_f_number: float = Field(alias="Image Space F/#")
    paraxial_working_f_number: float = Field(alias="Paraxial Working F/#")
    working_f_number: float = Field(alias="Working F/#")
    image_space_na: float = Field(alias="Image Space NA")
    object_space_na: float = Field(alias="Object Space NA")
    stop_radius: float = Field(alias="Stop Radius")
    paraxial_image_height: float = Field(alias="Paraxial Image Height")
    paraxial_magnification: float = Field(alias="Paraxial Magnification")
    entrance_pupil_diameter: float = Field(alias="Entrance Pupil Diameter")
    entrance_pupil_position: float = Field(alias="Entrance Pupil Position")
    exit_pupil_diameter: float = Field(alias="Exit Pupil Diameter")
    exit_pupil_position: float = Field(alias="Exit Pupil Position")
    field_type: str = Field(alias="Field Type")
    maximum_radial_field: float = Field(alias="Maximum Radial Field")
    primary_wavelength: float = Field(alias="Primary Wavelength")
    angular_magnification: float = Field(alias="Angular Magnification")
    lens_units: str = Field(alias="Lens Units")
    source_units: str = Field(alias="Source Units")
    analysis_units: str = Field(alias="Analysis Units")
    afocal_mode_units: str = Field(alias="Afocal Mode Units")
    mtf_units: str = Field(alias="MTF Units")
    include_calculated_data_in_session_file: bool = Field(alias="Include Calculated Data in Session File")


@analysis_result
class FieldData:
    number: int = Field(alias="#")
    x_value: float = Field(alias="X-Value")
    y_value: float = Field(alias="Y-Value")
    weight: float = Field(alias="Weight")


@analysis_result
class Fields:
    number_of_fields: int = Field(alias="Fields")
    field_type: str = Field(alias="Field Type")
    fields: list[FieldData] = Field(alias="Table")


@analysis_result
class VignettingData:
    number: int = Field(alias="#")
    vdx: float = Field(alias="VDX")
    vdy: float = Field(alias="VDY")
    vcx: float = Field(alias="VCX")
    vcy: float = Field(alias="VCY")
    van: float = Field(alias="VAN")


@analysis_result
class WavelengthData:
    number: int = Field(alias="#")
    value: float = Field(alias="Value")
    weight: float = Field(alias="Weight")


@analysis_result
class Wavelengths:
    number_of_wavelengths: int = Field(alias="Wavelengths")
    units: str = Field(alias="Units")
    wavelengths: list[WavelengthData] = Field(alias="Table")


@analysis_result
class ABCDMatrix:
    A: float
    B: float
    C: float
    D: float


@analysis_result
class SystemDataResult:
    """Data for the System Data analysis."""

    general_lens_data: GeneralLensData = Field(alias="General Lens Data")
    fields: Fields = Field(alias="Fields")
    vignetting: list[VignettingData] = Field(alias="Vignetting Factors")
    wavelengths: Wavelengths = Field(alias="Wavelengths")
    abcd_matrix: ABCDMatrix = Field(alias="Predicted coordinate ABCD matrix")


class SystemData(BaseAnalysisWrapper[SystemDataResult, None], analysis_type="SystemData", needs_text_output_file=True):
    """System Data analysis."""

    def __init__(self):
        """Create a new System Data analysis.

        This analysis does not require any settings.
        """
        super().__init__()

    def run_analysis(self) -> SystemDataResult:
        """Run the System Data analysis."""
        self.analysis.ApplyAndWaitForCompletion()

        return self.parse_output("system_data", transformer=SystemDataTransformer, result_type=SystemDataResult)



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = (
    "ApertureShapes",
    "ArrayMode",
    "BirefringentMode",
    "BirefringentReflections",
    "DetectorDataType",
    "DetectorShowAsType",
    "DiffractionSplitType",
    "DiffractiveFaceChoices",
    "DrawingResolutionType",
    "EfficiencySpectrumType",
    "EndCapOptions",
    "FaceIsType",
    "HologramTypes",
    "InterpolateChoices",
    "NCEIndexType",
    "ObjectColumn",
    "ObjectScatteringTypes",
    "ObjectType",
    "OrderChoices",
    "PixelAddressing",
    "PolarDetectorDataType",
    "RaysIgnoreObjectType",
    "RayTraceModes",
    "ScatterToType",
    "ShapeChoices",
    "SourceBulkScatterMode",
    "SourceColorMode",
    "SourceSamplingMethod",
    "UniformAngleChoices",
    "VolumePhysicsModelType",
)

class ApertureShapes:
    Annular: ApertureShapes = None
    Elliptical: ApertureShapes = None
    Rectangular: ApertureShapes = None
    CyliderWithRectangular: ApertureShapes = None

class ArrayMode:
    None_: ArrayMode = None
    Rectangular: ArrayMode = None
    Circular: ArrayMode = None
    Hexapolar: ArrayMode = None
    Hexagonal: ArrayMode = None

class BirefringentMode:
    TraceOrdinayAndExtraordinary: BirefringentMode = None
    TraceOrdinaryOnly: BirefringentMode = None
    TraceExtraordinaryOnly: BirefringentMode = None
    Waveplate: BirefringentMode = None

class BirefringentReflections:
    TraceReflectedAndRefracted: BirefringentReflections = None
    TraceRefractedOnly: BirefringentReflections = None
    TraceReflectedOnly: BirefringentReflections = None

class DetectorDataType:
    Real: DetectorDataType = None
    Imaginary: DetectorDataType = None
    Amplitude: DetectorDataType = None
    Power: DetectorDataType = None

class DetectorShowAsType:
    GreyScaleFlux: DetectorShowAsType = None
    InverseGreyScaleFlux: DetectorShowAsType = None
    FalseColorFlux: DetectorShowAsType = None
    InverseFalseColorFlux: DetectorShowAsType = None
    GreyScaleIrradiance: DetectorShowAsType = None
    InverseGreyScaleIrradiance: DetectorShowAsType = None
    FalseColorIrradiance: DetectorShowAsType = None
    InverseFalseColorIrradiance: DetectorShowAsType = None

class DiffractionSplitType:
    DontSplitByOrder: DiffractionSplitType = None
    SplitByTable: DiffractionSplitType = None
    SplitByDLL: DiffractionSplitType = None

class DiffractiveFaceChoices:
    FrontFace: DiffractiveFaceChoices = None
    BackFace: DiffractiveFaceChoices = None

class DrawingResolutionType:
    Standard: DrawingResolutionType = None
    Medium: DrawingResolutionType = None
    High: DrawingResolutionType = None
    Presentation: DrawingResolutionType = None
    Custom: DrawingResolutionType = None

class EfficiencySpectrumType:
    QuantumYield: EfficiencySpectrumType = None
    Excitation: EfficiencySpectrumType = None

class EndCapOptions:
    None_: EndCapOptions = None
    First: EndCapOptions = None
    Second: EndCapOptions = None
    Both: EndCapOptions = None

class FaceIsType:
    ObjectDefault: FaceIsType = None
    Reflective: FaceIsType = None
    Absorbing: FaceIsType = None

class HologramTypes:
    Type_1: HologramTypes = None
    Type_2: HologramTypes = None

class InterpolateChoices:
    Bicubic: InterpolateChoices = None
    Linear: InterpolateChoices = None

class NCEIndexType:
    Isotropic: NCEIndexType = None
    Birefringent: NCEIndexType = None
    GRIN: NCEIndexType = None

class ObjectColumn:
    Comment: ObjectColumn = None
    RefObject: ObjectColumn = None
    InsideOf: ObjectColumn = None
    XPosition: ObjectColumn = None
    YPosition: ObjectColumn = None
    ZPosition: ObjectColumn = None
    TiltX: ObjectColumn = None
    TiltY: ObjectColumn = None
    TiltZ: ObjectColumn = None
    Material: ObjectColumn = None
    Par1: ObjectColumn = None
    Par2: ObjectColumn = None
    Par3: ObjectColumn = None
    Par4: ObjectColumn = None
    Par5: ObjectColumn = None
    Par6: ObjectColumn = None
    Par7: ObjectColumn = None
    Par8: ObjectColumn = None
    Par9: ObjectColumn = None
    Par10: ObjectColumn = None
    Par11: ObjectColumn = None
    Par12: ObjectColumn = None
    Par13: ObjectColumn = None
    Par14: ObjectColumn = None
    Par15: ObjectColumn = None
    Par16: ObjectColumn = None
    Par17: ObjectColumn = None
    Par18: ObjectColumn = None
    Par19: ObjectColumn = None
    Par20: ObjectColumn = None
    Par21: ObjectColumn = None
    Par22: ObjectColumn = None
    Par23: ObjectColumn = None
    Par24: ObjectColumn = None
    Par25: ObjectColumn = None
    Par26: ObjectColumn = None
    Par27: ObjectColumn = None
    Par28: ObjectColumn = None
    Par29: ObjectColumn = None
    Par30: ObjectColumn = None
    Par31: ObjectColumn = None
    Par32: ObjectColumn = None
    Par33: ObjectColumn = None
    Par34: ObjectColumn = None
    Par35: ObjectColumn = None
    Par36: ObjectColumn = None
    Par37: ObjectColumn = None
    Par38: ObjectColumn = None
    Par39: ObjectColumn = None
    Par40: ObjectColumn = None
    Par41: ObjectColumn = None
    Par42: ObjectColumn = None
    Par43: ObjectColumn = None
    Par44: ObjectColumn = None
    Par45: ObjectColumn = None
    Par46: ObjectColumn = None
    Par47: ObjectColumn = None
    Par48: ObjectColumn = None
    Par49: ObjectColumn = None
    Par50: ObjectColumn = None
    Par51: ObjectColumn = None
    Par52: ObjectColumn = None
    Par53: ObjectColumn = None
    Par54: ObjectColumn = None
    Par55: ObjectColumn = None
    Par56: ObjectColumn = None
    Par57: ObjectColumn = None
    Par58: ObjectColumn = None
    Par59: ObjectColumn = None
    Par60: ObjectColumn = None
    Par61: ObjectColumn = None
    Par62: ObjectColumn = None
    Par63: ObjectColumn = None
    Par64: ObjectColumn = None
    Par65: ObjectColumn = None
    Par66: ObjectColumn = None
    Par67: ObjectColumn = None
    Par68: ObjectColumn = None
    Par69: ObjectColumn = None
    Par70: ObjectColumn = None
    Par71: ObjectColumn = None
    Par72: ObjectColumn = None
    Par73: ObjectColumn = None
    Par74: ObjectColumn = None
    Par75: ObjectColumn = None
    Par76: ObjectColumn = None
    Par77: ObjectColumn = None
    Par78: ObjectColumn = None
    Par79: ObjectColumn = None
    Par80: ObjectColumn = None
    Par81: ObjectColumn = None
    Par82: ObjectColumn = None
    Par83: ObjectColumn = None
    Par84: ObjectColumn = None
    Par85: ObjectColumn = None
    Par86: ObjectColumn = None
    Par87: ObjectColumn = None
    Par88: ObjectColumn = None
    Par89: ObjectColumn = None
    Par90: ObjectColumn = None
    Par91: ObjectColumn = None
    Par92: ObjectColumn = None
    Par93: ObjectColumn = None
    Par94: ObjectColumn = None
    Par95: ObjectColumn = None
    Par96: ObjectColumn = None
    Par97: ObjectColumn = None
    Par98: ObjectColumn = None
    Par99: ObjectColumn = None
    Par100: ObjectColumn = None
    Par101: ObjectColumn = None
    Par102: ObjectColumn = None
    Par103: ObjectColumn = None
    Par104: ObjectColumn = None
    Par105: ObjectColumn = None
    Par106: ObjectColumn = None
    Par107: ObjectColumn = None
    Par108: ObjectColumn = None
    Par109: ObjectColumn = None
    Par110: ObjectColumn = None
    Par111: ObjectColumn = None
    Par112: ObjectColumn = None
    Par113: ObjectColumn = None
    Par114: ObjectColumn = None
    Par115: ObjectColumn = None
    Par116: ObjectColumn = None
    Par117: ObjectColumn = None
    Par118: ObjectColumn = None
    Par119: ObjectColumn = None
    Par120: ObjectColumn = None
    Par121: ObjectColumn = None
    Par122: ObjectColumn = None
    Par123: ObjectColumn = None
    Par124: ObjectColumn = None
    Par125: ObjectColumn = None
    Par126: ObjectColumn = None
    Par127: ObjectColumn = None
    Par128: ObjectColumn = None
    Par129: ObjectColumn = None
    Par130: ObjectColumn = None
    Par131: ObjectColumn = None
    Par132: ObjectColumn = None
    Par133: ObjectColumn = None
    Par134: ObjectColumn = None
    Par135: ObjectColumn = None
    Par136: ObjectColumn = None
    Par137: ObjectColumn = None
    Par138: ObjectColumn = None
    Par139: ObjectColumn = None
    Par140: ObjectColumn = None
    Par141: ObjectColumn = None
    Par142: ObjectColumn = None
    Par143: ObjectColumn = None
    Par144: ObjectColumn = None
    Par145: ObjectColumn = None
    Par146: ObjectColumn = None
    Par147: ObjectColumn = None
    Par148: ObjectColumn = None
    Par149: ObjectColumn = None
    Par150: ObjectColumn = None
    Par151: ObjectColumn = None
    Par152: ObjectColumn = None
    Par153: ObjectColumn = None
    Par154: ObjectColumn = None
    Par155: ObjectColumn = None
    Par156: ObjectColumn = None
    Par157: ObjectColumn = None
    Par158: ObjectColumn = None
    Par159: ObjectColumn = None
    Par160: ObjectColumn = None
    Par161: ObjectColumn = None
    Par162: ObjectColumn = None
    Par163: ObjectColumn = None
    Par164: ObjectColumn = None
    Par165: ObjectColumn = None
    Par166: ObjectColumn = None
    Par167: ObjectColumn = None
    Par168: ObjectColumn = None
    Par169: ObjectColumn = None
    Par170: ObjectColumn = None
    Par171: ObjectColumn = None
    Par172: ObjectColumn = None
    Par173: ObjectColumn = None
    Par174: ObjectColumn = None
    Par175: ObjectColumn = None
    Par176: ObjectColumn = None
    Par177: ObjectColumn = None
    Par178: ObjectColumn = None
    Par179: ObjectColumn = None
    Par180: ObjectColumn = None
    Par181: ObjectColumn = None
    Par182: ObjectColumn = None
    Par183: ObjectColumn = None
    Par184: ObjectColumn = None
    Par185: ObjectColumn = None
    Par186: ObjectColumn = None
    Par187: ObjectColumn = None
    Par188: ObjectColumn = None
    Par189: ObjectColumn = None
    Par190: ObjectColumn = None
    Par191: ObjectColumn = None
    Par192: ObjectColumn = None
    Par193: ObjectColumn = None
    Par194: ObjectColumn = None
    Par195: ObjectColumn = None
    Par196: ObjectColumn = None
    Par197: ObjectColumn = None
    Par198: ObjectColumn = None
    Par199: ObjectColumn = None
    Par200: ObjectColumn = None
    Par201: ObjectColumn = None
    Par202: ObjectColumn = None
    Par203: ObjectColumn = None
    Par204: ObjectColumn = None
    Par205: ObjectColumn = None
    Par206: ObjectColumn = None
    Par207: ObjectColumn = None
    Par208: ObjectColumn = None
    Par209: ObjectColumn = None
    Par210: ObjectColumn = None
    Par211: ObjectColumn = None
    Par212: ObjectColumn = None
    Par213: ObjectColumn = None
    Par214: ObjectColumn = None
    Par215: ObjectColumn = None
    Par216: ObjectColumn = None
    Par217: ObjectColumn = None
    Par218: ObjectColumn = None
    Par219: ObjectColumn = None
    Par220: ObjectColumn = None
    Par221: ObjectColumn = None
    Par222: ObjectColumn = None
    Par223: ObjectColumn = None
    Par224: ObjectColumn = None
    Par225: ObjectColumn = None
    Par226: ObjectColumn = None
    Par227: ObjectColumn = None
    Par228: ObjectColumn = None
    Par229: ObjectColumn = None
    Par230: ObjectColumn = None
    Par231: ObjectColumn = None
    Par232: ObjectColumn = None
    Par233: ObjectColumn = None
    Par234: ObjectColumn = None
    Par235: ObjectColumn = None
    Par236: ObjectColumn = None
    Par237: ObjectColumn = None
    Par238: ObjectColumn = None
    Par239: ObjectColumn = None
    Par240: ObjectColumn = None
    Par241: ObjectColumn = None
    Par242: ObjectColumn = None
    Par243: ObjectColumn = None
    Par244: ObjectColumn = None
    Par245: ObjectColumn = None
    Par246: ObjectColumn = None
    Par247: ObjectColumn = None
    Par248: ObjectColumn = None
    Par249: ObjectColumn = None
    Par250: ObjectColumn = None

class ObjectScatteringTypes:
    None_: ObjectScatteringTypes = None
    Lambertian: ObjectScatteringTypes = None
    Gaussian: ObjectScatteringTypes = None
    ABg: ObjectScatteringTypes = None
    ABgFile: ObjectScatteringTypes = None
    BSDF: ObjectScatteringTypes = None
    ISScatterCatalog: ObjectScatteringTypes = None
    User: ObjectScatteringTypes = None

class ObjectType:
    AnnularAsphericLens: ObjectType = None
    AnnularAxialLens: ObjectType = None
    AnnularVolume: ObjectType = None
    Annulus: ObjectType = None
    Array: ObjectType = None
    ArrayRing: ObjectType = None
    AsphericSurface: ObjectType = None
    AsphericSurface2: ObjectType = None
    AxiconSurface: ObjectType = None
    BiconicLens: ObjectType = None
    BiconicSurface: ObjectType = None
    BiconicZernike: ObjectType = None
    BiconicZernikeSurface: ObjectType = None
    Binary1: ObjectType = None
    Binary2: ObjectType = None
    Binary2A: ObjectType = None
    Boolean: ObjectType = None
    BooleanCAD: ObjectType = None
    CADAssemblyAutodeskInventor: ObjectType = None
    CADAssemblyCreoParametric: ObjectType = None
    CADAssemblySolidWorks: ObjectType = None
    CADPartAutodeskInventor: ObjectType = None
    CADPartCreoParametric: ObjectType = None
    CADPartSolidWorks: ObjectType = None
    CADPartSTEPIGESSAT: ObjectType = None
    CADPartSTL: ObjectType = None
    CADPartZPD: ObjectType = None
    Cone: ObjectType = None
    CPC: ObjectType = None
    CPCRectangular: ObjectType = None
    CylinderPipe: ObjectType = None
    CylinderVolume: ObjectType = None
    Cylinder2Pipe: ObjectType = None
    Cylinder2Volume: ObjectType = None
    DetectorColor: ObjectType = None
    DetectorPolar: ObjectType = None
    DetectorRectangle: ObjectType = None
    DetectorSurface: ObjectType = None
    DetectorVolume: ObjectType = None
    DiffractionGrating: ObjectType = None
    DualBEFSurface: ObjectType = None
    Ellipse: ObjectType = None
    EllipticalVolume: ObjectType = None
    EvenAsphereLens: ObjectType = None
    ExtendedOddAsphereLens: ObjectType = None
    ExtendedPolynomialLens: ObjectType = None
    ExtendedPolynomialSurface: ObjectType = None
    Extruded: ObjectType = None
    FacetedSurface: ObjectType = None
    FreeformZ: ObjectType = None
    Fresnel1: ObjectType = None
    Fresnel2: ObjectType = None
    GridSagLens: ObjectType = None
    GridSagSurface: ObjectType = None
    GridSagFrontBack: ObjectType = None
    HexagonalLensletArray: ObjectType = None
    HologramLens: ObjectType = None
    HologramSurface: ObjectType = None
    JonesMatrix: ObjectType = None
    LensletArray1: ObjectType = None
    LensletArray2: ObjectType = None
    MEMS: ObjectType = None
    NullObject: ObjectType = None
    OddAsphereLens: ObjectType = None
    ParaxialLens: ObjectType = None
    PolygonObject: ObjectType = None
    RayRotator: ObjectType = None
    RectangularCorner: ObjectType = None
    Rectangle: ObjectType = None
    RectangularPipe: ObjectType = None
    RectangularPipeGrating: ObjectType = None
    RectangularRoof: ObjectType = None
    RectangularTorusSurface: ObjectType = None
    RectangularTorusVolume: ObjectType = None
    RectangularVolume: ObjectType = None
    RectangularVolumeGrating: ObjectType = None
    ReverseRadianceDetector: ObjectType = None
    ReverseRadianceTarget: ObjectType = None
    Slide: ObjectType = None
    SourceDiffractive: ObjectType = None
    SourceDiode: ObjectType = None
    SourceDLL: ObjectType = None
    SourceEllipse: ObjectType = None
    SourceEULUMDATFile: ObjectType = None
    SourceFilament: ObjectType = None
    SourceFile: ObjectType = None
    SourceGaussian: ObjectType = None
    SourceIESNAFile: ObjectType = None
    SourceImported: ObjectType = None
    SourceObject: ObjectType = None
    SourcePoint: ObjectType = None
    SourceRadial: ObjectType = None
    SourceRay: ObjectType = None
    SourceRectangle: ObjectType = None
    SourceTube: ObjectType = None
    SourceTwoAngle: ObjectType = None
    SourceVolumeCylindrical: ObjectType = None
    SourceVolumeElliptical: ObjectType = None
    SourceVolumeRectangular: ObjectType = None
    Sphere: ObjectType = None
    StandardLens: ObjectType = None
    StandardSurface: ObjectType = None
    Swept: ObjectType = None
    TabulatedFactedRadial: ObjectType = None
    TabulatedFactedToroid: ObjectType = None
    TabulatedFrenselRadial: ObjectType = None
    ToroidalHologram: ObjectType = None
    ToroidalLens: ObjectType = None
    ToroidalSurface: ObjectType = None
    ToroidalSurfaceOddASphere: ObjectType = None
    TorusSurface: ObjectType = None
    TorusVolume: ObjectType = None
    TriangularCorner: ObjectType = None
    Triangle: ObjectType = None
    UserDefinedObject: ObjectType = None
    WolterSurface: ObjectType = None
    ZernikeSurface: ObjectType = None
    BooleanNative: ObjectType = None
    CompoundLens: ObjectType = None
    QTypeAsphereSurface: ObjectType = None
    OffAxisMirror: ObjectType = None

class OrderChoices:
    Before: OrderChoices = None
    After: OrderChoices = None

class PixelAddressing:
    ByRow: PixelAddressing = None
    ByColumn: PixelAddressing = None
    Individually: PixelAddressing = None

class PolarDetectorDataType:
    Power: PolarDetectorDataType = None
    PowerSolidAngle: PolarDetectorDataType = None
    Lumens: PolarDetectorDataType = None
    LumensSolidAngle: PolarDetectorDataType = None
    Cx: PolarDetectorDataType = None
    Cy: PolarDetectorDataType = None
    u_T: PolarDetectorDataType = None
    u_V: PolarDetectorDataType = None
    TriX: PolarDetectorDataType = None
    TriY: PolarDetectorDataType = None
    TriZ: PolarDetectorDataType = None

class RaysIgnoreObjectType:
    Never: RaysIgnoreObjectType = None
    Always: RaysIgnoreObjectType = None
    OnLaunch: RaysIgnoreObjectType = None

class RayTraceModes:
    Standard: RayTraceModes = None
    Flat: RayTraceModes = None
    Shaded: RayTraceModes = None
    Kernel: RayTraceModes = None

class ScatterToType:
    ScatterToList: ScatterToType = None
    ImportanceSampling: ScatterToType = None

class ShapeChoices:
    Rectangular: ShapeChoices = None
    Elliptical: ShapeChoices = None

class SourceBulkScatterMode:
    Many: SourceBulkScatterMode = None
    Once: SourceBulkScatterMode = None
    Never: SourceBulkScatterMode = None

class SourceColorMode:
    SystemWavelengths: SourceColorMode = None
    CIE1931Tristimulus: SourceColorMode = None
    CIE1931Chromaticity: SourceColorMode = None
    CIE1931RGB: SourceColorMode = None
    UniformPowerSpectrum: SourceColorMode = None
    D65White: SourceColorMode = None
    ColorTemperature: SourceColorMode = None
    BlackBodySpectrum: SourceColorMode = None
    SpectrumFile: SourceColorMode = None
    CIE1976: SourceColorMode = None

class SourceSamplingMethod:
    Random: SourceSamplingMethod = None
    Sobol: SourceSamplingMethod = None

class UniformAngleChoices:
    UniformIrradiance: UniformAngleChoices = None
    UniformInAngleSpace: UniformAngleChoices = None

class VolumePhysicsModelType:
    None_: VolumePhysicsModelType = None
    AngleScattering: VolumePhysicsModelType = None
    DLLDefinedScattering: VolumePhysicsModelType = None
    PhotoluminescenceModel: VolumePhysicsModelType = None



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = (
    "ACISExportVersion",
    "ArchiveFileStatus",
    "CADAngularToleranceType",
    "CADFileType",
    "CADToleranceType",
    "DataTypes",
    "EntryCompressionModes",
    "LensShape",
    "LensType",
    "QuickAdjustCriterion",
    "QuickAdjustType",
    "QuickFocusCriterion",
    "RayPatternType",
    "ScaleToUnits",
    "SplineSegmentsType",
    "ZemaxFileTypes",
)

class ACISExportVersion:
    Current: ACISExportVersion = None
    V25: ACISExportVersion = None
    V26: ACISExportVersion = None
    V27: ACISExportVersion = None
    V28: ACISExportVersion = None
    V29: ACISExportVersion = None
    V30: ACISExportVersion = None

class ArchiveFileStatus:
    Okay: ArchiveFileStatus = None
    InvalidVersion: ArchiveFileStatus = None
    InvalidFile: ArchiveFileStatus = None
    UnableToOpen: ArchiveFileStatus = None

class CADAngularToleranceType:
    Low: CADAngularToleranceType = None
    Medium: CADAngularToleranceType = None
    High: CADAngularToleranceType = None
    Presentation: CADAngularToleranceType = None

class CADFileType:
    IGES: CADFileType = None
    STEP: CADFileType = None
    SAT: CADFileType = None
    STL: CADFileType = None
    SAB: CADFileType = None
    ASAT: CADFileType = None
    ASAB: CADFileType = None
    MODEL: CADFileType = None
    CATPART: CADFileType = None
    CATPRODUCT: CADFileType = None
    XCGM: CADFileType = None
    ZMO: CADFileType = None
    XT: CADFileType = None
    XB: CADFileType = None
    PRC: CADFileType = None
    JT: CADFileType = None
    N3MF: CADFileType = None
    U3D: CADFileType = None
    VRML: CADFileType = None
    OBJ: CADFileType = None

class CADToleranceType:
    N_TenEMinus4: CADToleranceType = None
    N_TenEMinus5: CADToleranceType = None
    N_TenEMinus6: CADToleranceType = None
    N_TenEMinus7: CADToleranceType = None

class DataTypes:
    NotSet: DataTypes = None
    Boolean: DataTypes = None
    Integer: DataTypes = None
    IntegerArray: DataTypes = None
    IntegerMatrix: DataTypes = None
    Float: DataTypes = None
    FloatArray: DataTypes = None
    FloatMatrix: DataTypes = None
    Double: DataTypes = None
    DoubleArray: DataTypes = None
    DoubleMatrix: DataTypes = None
    String: DataTypes = None
    StringArray: DataTypes = None
    StringMatrix: DataTypes = None
    ByteArray: DataTypes = None
    Dictionary: DataTypes = None
    Serializable: DataTypes = None
    File: DataTypes = None

class EntryCompressionModes:
    Auto: EntryCompressionModes = None
    On: EntryCompressionModes = None
    Off: EntryCompressionModes = None

class LensShape:
    Unknown: LensShape = None
    Equi: LensShape = None
    Bi: LensShape = None
    Plano: LensShape = None
    Meniscus: LensShape = None

class LensType:
    Other: LensType = None
    Spherical: LensType = None
    GRIN: LensType = None
    Aspheric: LensType = None
    Toroidal: LensType = None

class QuickAdjustCriterion:
    SpotSizeRadial: QuickAdjustCriterion = None
    SpotSizeXOnly: QuickAdjustCriterion = None
    SpotSizeYOnly: QuickAdjustCriterion = None
    AngularRadial: QuickAdjustCriterion = None
    AngularXOnly: QuickAdjustCriterion = None
    AngularYOnly: QuickAdjustCriterion = None

class QuickAdjustType:
    Radius: QuickAdjustType = None
    Thickness: QuickAdjustType = None

class QuickFocusCriterion:
    SpotSizeRadial: QuickFocusCriterion = None
    SpotSizeXOnly: QuickFocusCriterion = None
    SpotSizeYOnly: QuickFocusCriterion = None
    RMSWavefront: QuickFocusCriterion = None

class RayPatternType:
    XYFan: RayPatternType = None
    XFan: RayPatternType = None
    YFan: RayPatternType = None
    Ring: RayPatternType = None
    List: RayPatternType = None
    Random: RayPatternType = None
    Grid: RayPatternType = None
    SolidRing: RayPatternType = None

class ScaleToUnits:
    Millimeters: ScaleToUnits = None
    Centimeters: ScaleToUnits = None
    Inches: ScaleToUnits = None
    Meters: ScaleToUnits = None

class SplineSegmentsType:
    N_016: SplineSegmentsType = None
    N_032: SplineSegmentsType = None
    N_064: SplineSegmentsType = None
    N_128: SplineSegmentsType = None
    N_256: SplineSegmentsType = None
    N_512: SplineSegmentsType = None

class ZemaxFileTypes:
    Unknown: ZemaxFileTypes = None
    LMX: ZemaxFileTypes = None
    User: ZemaxFileTypes = None



"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("OptimizationAlgorithm", "OptimizationCycles", "OptimizationSaveCount")

class OptimizationAlgorithm:
    DampedLeastSquares: OptimizationAlgorithm = None
    OrthogonalDescent: OptimizationAlgorithm = None

class OptimizationCycles:
    Automatic: OptimizationCycles = None
    Fixed_1_Cycle: OptimizationCycles = None
    Fixed_5_Cycles: OptimizationCycles = None
    Fixed_10_Cycles: OptimizationCycles = None
    Fixed_50_Cycles: OptimizationCycles = None
    Infinite: OptimizationCycles = None

class OptimizationSaveCount:
    Save_10: OptimizationSaveCount = None
    Save_20: OptimizationSaveCount = None
    Save_30: OptimizationSaveCount = None
    Save_40: OptimizationSaveCount = None
    Save_50: OptimizationSaveCount = None
    Save_60: OptimizationSaveCount = None
    Save_70: OptimizationSaveCount = None
    Save_80: OptimizationSaveCount = None
    Save_90: OptimizationSaveCount = None
    Save_100: OptimizationSaveCount = None
